/*
 * commit <password-file> <index-file> <count-file>
 * Check password, add flag to index file, and increment count.
 *
 * Ex:
 * echo "password flag" | ./commit key.h index.html count.txt [<commands...>]
 *
 * Password file format:
 * "PASSWORD:abcdefg"
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <fcntl.h>
#include <unistd.h>

void error_exit(char *message)
{
  fprintf(stderr, "%s\n", message);
  exit(1);
}

char *strchrstr(char *s, char *cs)
{
  int i;
  for (; *s; s++) {
    for (i = 0; cs[i]; i++) {
      if (*s == cs[i])
	return s;
    }
  }
  return NULL;
}

char *strcheck(char *s)
{
  for (; *s; s++) {
    if (!isdigit(*s) && !isalpha(*s))
      return s;
  }
  return NULL;
}

int main(int argc, char *argv[], char *env[])
{
  int pfd, ifd, cfd, size, count, i, succeeded = 0;
  char *password_file, *index_file, *count_file;
  char password[256], buffer[256];
  char *p, *input_password, *input_flag;

  argc--; argv++;
  password_file = argv[0]; argc--; argv++;
  index_file    = argv[0]; argc--; argv++;
  count_file    = argv[0]; argc--; argv++;

  pfd = open(password_file, O_RDONLY);
  if (pfd < 0) {
    write(1, "Updating files... wait for seconds.\n", 36);
    exit(0);
  }
  size = read(pfd, password, sizeof(password));
  if (size < 0)
    error_exit("Password read error.");
  password[size] = '\0';
  close(pfd);

  while ((p = strchr(password, '\n')) != NULL)
    *p = '\0';

  while ((p = strchr(password, '\"')) != NULL)
    memmove(p, p + 1, strlen(password) + 1 - ((p + 1) - password));

  if (!strncmp(password, "PASSWORD:", 9))
    memmove(password, password + 9, strlen(password) + 1 - 9);

  size = read(0, buffer, sizeof(buffer));
  if (size < 0)
    error_exit("Read error.");
  buffer[size] = '\0';

  while ((p = strchr(buffer, '\n')) != NULL)
    *p = '\0';

  for (p = buffer; *p && isspace(*p); p++)
    ;
  input_password = p;
  for (; *p && !isspace(*p); p++)
    ;
  if (*p)
    *(p++) = '\0';
  for (; *p && isspace(*p); p++)
    ;
  input_flag = p;
  for (; *p && !isspace(*p); p++)
    ;
  if (*p)
    *(p++) = '\0';

  if (!strncmp(input_password, "PASSWORD:", 9))
    input_password += 9;

  while ((p = strcheck(input_flag)) != NULL)
    memmove(p, p + 1, strlen(input_flag) + 1 - ((p + 1) - input_flag));

  if (strcmp(password, input_password)) {
    write(1, "Invalid password.\n", 18);
    goto end;
  }
  write(1, "Valid password.\n", 16);
  succeeded = 1;

  ifd = open(index_file, O_RDWR|O_APPEND);
  if (ifd < 0)
    error_exit("Index file not found.");
  write(ifd, input_flag, strlen(input_flag));
  write(ifd, "\n", 1);
  close(ifd);

  cfd = open(count_file, O_RDWR, 0644);
  if (cfd < 0)
    error_exit("Count file not found.");

  size = read(cfd, buffer, sizeof(buffer));
  if (size < 0)
    error_exit("Read error.");
  buffer[size] = '\0';

  for (p = buffer; *p && isspace(*p); p++)
    ;
  count = atoi(p);

  count++;

  sprintf(buffer, "%d\n", count);
  lseek(cfd, 0, SEEK_SET);

  /*
   * Consider the other server by fork read/write count file here,
   * write by atomic.
   */
  write(cfd, buffer, strlen(buffer));
#if 0
  ftruncate(cfd, strlen(buffer));
#endif

  close(cfd);

end:
  for (i = 0; i < argc; i++) {
    if (!strcmp(argv[i], "RESULT"))
      argv[i] = succeeded ? "1" : "0";
  }

  if (argc > 0) {
    execve(argv[0], argv, NULL);
  }

  return 0;
}
