--- sim/aarch64/simulator.c.orig	2017-01-21 22:48:42.000000000 +0900
+++ sim/aarch64/simulator.c	2018-12-20 22:50:45.129961000 +0900
@@ -12445,6 +12445,178 @@
   aarch64_set_reg_u64 (cpu, rd, NO_SP, result);
 }
 
+#define RANDOM_INSTRUCTION
+#ifdef RANDOM_INSTRUCTION
+#define RANDOM_INSTRUCTION_LIMIT 100
+#ifdef RANDOM_INSTRUCTION_LIMIT
+static void limit_check()
+{
+  static int count = 0, limit = RANDOM_INSTRUCTION_LIMIT;
+  int fd;
+  char buf[128];
+  if (!count) {
+    count++;
+    fd = open("RANDOM_INSTRUCTION_LIMIT", 0);
+    if (fd >= 0) {
+      if (read(fd, buf, sizeof(buf)) > 0)
+	limit = atoi(buf);
+      close(fd);
+      if (!limit) count = -1;
+    }
+  }
+  if ((count > 0) && (count++ > limit)) {
+    fprintf(stderr, "Limit of special instruction. (%d)\n", limit);
+    exit(1);
+  }
+}
+#endif
+
+typedef enum {
+  XORSHIFT_SEQ_LRL = 0,
+  XORSHIFT_SEQ_RLR = 1,
+  XORSHIFT_SEQ_LLR = 2,
+  XORSHIFT_SEQ_RRL = 3
+} xorshift_seq_t;
+
+/* xorshift (32bit) */
+uint32_t xorshift32_param = 0;
+uint32_t xorshift32_value = 0;
+
+static void
+xorshift32param (sim_cpu *cpu)
+{
+  unsigned rn = INSTR (9, 5);
+  unsigned rd = INSTR (4, 0);
+  uint32_t value = aarch64_get_reg_u32 (cpu, rn, NO_SP);
+  uint32_t result = 0;
+
+  result = xorshift32_param;
+  xorshift32_param = (value & ((1 << 17) - 1));
+
+  aarch64_set_reg_u64 (cpu, rd, NO_SP, result);
+}
+
+static void
+xorshift32seed (sim_cpu *cpu)
+{
+  unsigned rn = INSTR (9, 5);
+  unsigned rd = INSTR (4, 0);
+  uint32_t value = aarch64_get_reg_u32 (cpu, rn, NO_SP);
+  uint32_t result = 0;
+
+  result = xorshift32_value;
+  xorshift32_value = value;
+
+  aarch64_set_reg_u64 (cpu, rd, NO_SP, result);
+}
+
+static void
+xorshift32value (sim_cpu *cpu)
+{
+  unsigned rn = INSTR (9, 5);
+  unsigned rd = INSTR (4, 0);
+  uint32_t v = aarch64_get_reg_u32 (cpu, rn, NO_SP);
+  uint32_t result = 0;
+  int seq, a, b, c;
+
+#ifdef RANDOM_INSTRUCTION_LIMIT
+  limit_check();
+#endif
+
+  seq = xorshift32_param & 3;
+
+  a = (xorshift32_param >>  2) & 0x1f;
+  b = (xorshift32_param >>  7) & 0x1f;
+  c = (xorshift32_param >> 12) & 0x1f;
+  a = a ? a :  1;
+  b = b ? b :  3;
+  c = c ? c : 10;
+
+  v = v ? v : xorshift32_value;
+  v = v ? v : 0xbadbeef;
+
+  switch (seq) {
+  case XORSHIFT_SEQ_LRL: v = v ^ (v<<a); v = v ^ (v>>b); v = v ^ (v<<c); break;
+  case XORSHIFT_SEQ_RLR: v = v ^ (v>>a); v = v ^ (v<<b); v = v ^ (v>>c); break;
+  case XORSHIFT_SEQ_LLR: v = v ^ (v<<a); v = v ^ (v<<c); v = v ^ (v>>b); break;
+  case XORSHIFT_SEQ_RRL: v = v ^ (v>>a); v = v ^ (v>>c); v = v ^ (v<<b); break;
+  default: break;
+  }
+  xorshift32_value = v;
+  result = v;
+  aarch64_set_reg_u64 (cpu, rd, NO_SP, result);
+}
+
+/* xorshift (64bit) */
+uint64_t xorshift64_param = 0;
+uint64_t xorshift64_value = 0;
+
+static void
+xorshift64param (sim_cpu *cpu)
+{
+  unsigned rn = INSTR (9, 5);
+  unsigned rd = INSTR (4, 0);
+  uint64_t value = aarch64_get_reg_u64 (cpu, rn, NO_SP);
+  uint64_t result = 0;
+
+  result = xorshift64_param;
+  xorshift64_param = (value & ((1 << 20) - 1));
+
+  aarch64_set_reg_u64 (cpu, rd, NO_SP, result);
+}
+
+static void
+xorshift64seed (sim_cpu *cpu)
+{
+  unsigned rn = INSTR (9, 5);
+  unsigned rd = INSTR (4, 0);
+  uint64_t value = aarch64_get_reg_u64 (cpu, rn, NO_SP);
+  uint64_t result = 0;
+
+  result = xorshift64_value;
+  xorshift64_value = value;
+
+  aarch64_set_reg_u64 (cpu, rd, NO_SP, result);
+}
+
+static void
+xorshift64value (sim_cpu *cpu)
+{
+  unsigned rn = INSTR (9, 5);
+  unsigned rd = INSTR (4, 0);
+  uint64_t v = aarch64_get_reg_u64 (cpu, rn, NO_SP);
+  uint64_t result = 0;
+  int seq, a, b, c;
+
+#ifdef RANDOM_INSTRUCTION_LIMIT
+  limit_check();
+#endif
+
+  seq = xorshift64_param & 3;
+
+  a = (xorshift64_param >>  2) & 0x3f;
+  b = (xorshift64_param >>  8) & 0x3f;
+  c = (xorshift64_param >> 14) & 0x3f;
+  a = a ? a : 34;
+  b = b ? b : 41;
+  c = c ? c : 23;
+
+  v = v ? v : xorshift64_value;
+  v = v ? v : 0xbadbadbadbadbeef;
+
+  switch (seq) {
+  case XORSHIFT_SEQ_LRL: v = v ^ (v<<a); v = v ^ (v>>b); v = v ^ (v<<c); break;
+  case XORSHIFT_SEQ_RLR: v = v ^ (v>>a); v = v ^ (v<<b); v = v ^ (v>>c); break;
+  case XORSHIFT_SEQ_LLR: v = v ^ (v<<a); v = v ^ (v<<c); v = v ^ (v>>b); break;
+  case XORSHIFT_SEQ_RRL: v = v ^ (v>>a); v = v ^ (v>>c); v = v ^ (v<<b); break;
+  default: break;
+  }
+  xorshift64_value = v;
+  result = v;
+  aarch64_set_reg_u64 (cpu, rd, NO_SP, result);
+}
+#endif
+
 static void
 dexDataProc1Source (sim_cpu *cpu)
 {
@@ -12471,7 +12643,11 @@
   if (opcode2 != 0)
     HALT_UNALLOC;
 
+#ifdef RANDOM_INSTRUCTION
+  if (opcode & 0x20)
+#else
   if (opcode & 0x38)
+#endif
     HALT_UNALLOC;
 
   switch (dispatch)
@@ -12481,12 +12657,23 @@
     case 2: rev32 (cpu); return;
     case 4: clz32 (cpu); return;
     case 5: cls32 (cpu); return;
+#ifdef RANDOM_INSTRUCTION
+    case 6: xorshift32value (cpu); return;
+    case 7: xorshift32seed (cpu); return;
+#endif
     case 8: rbit64 (cpu); return;
     case 9: revh64 (cpu); return;
     case 10:rev32 (cpu); return;
     case 11:rev64 (cpu); return;
     case 12:clz64 (cpu); return;
     case 13:cls64 (cpu); return;
+#ifdef RANDOM_INSTRUCTION
+    case 14: xorshift64value (cpu); return;
+    case 15: xorshift64seed (cpu); return;
+
+    case 16: xorshift32param (cpu); return;
+    case 24: xorshift64param (cpu); return;
+#endif
     default: HALT_UNALLOC;
     }
 }
@@ -13626,7 +13813,14 @@
 	  }
 	else if (fd == 1)
 	  {
+#define USE_WRITE
+#ifndef USE_WRITE
 	    printf ("%.*s", (int) len, aarch64_get_mem_ptr (cpu, buf));
+#else
+	    /* for output \0 */
+	    result = fwrite (aarch64_get_mem_ptr (cpu, buf), (int) len, 1, stdout);
+	    fflush(stdout);
+#endif
 	  }
 	else if (fd == 2)
 	  {
