--- sim/arm/armemu.c.orig	2016-08-02 00:50:21.000000000 +0900
+++ sim/arm/armemu.c	2018-12-13 22:44:06.918698000 +0900
@@ -1138,6 +1138,92 @@
     }
 }
 
+#ifdef MODE32
+#define RANDOM_INSTRUCTION
+#endif
+#ifdef RANDOM_INSTRUCTION
+#define RANDOM_INSTRUCTION_LIMIT 100
+#ifdef RANDOM_INSTRUCTION_LIMIT
+static void limit_check()
+{
+  static int count = 0, limit = RANDOM_INSTRUCTION_LIMIT;
+  int fd;
+  char buf[128];
+  if (!count) {
+    count++;
+    fd = open("RANDOM_INSTRUCTION_LIMIT", 0);
+    if (fd >= 0) {
+      if (read(fd, buf, sizeof(buf)) > 0)
+	limit = atoi(buf);
+      close(fd);
+      if (!limit) count = -1;
+    }
+  }
+  if ((count > 0) && (count++ > limit)) {
+    fprintf(stderr, "Limit of special instruction. (%d)\n", limit);
+    exit(1);
+  }
+}
+#endif
+
+/* Linear (32bit) */
+uint32_t linear32_param0 = 0;
+uint32_t linear32_param1 = 0;
+uint32_t linear32_value  = 0;
+
+static uint32_t
+linear32param0 (uint32_t value)
+{
+  uint32_t result;
+  result = linear32_param0;
+  linear32_param0 = value;
+  return result;
+}
+
+static uint32_t
+linear32param1 (uint32_t value)
+{
+  uint32_t result;
+  result = linear32_param1;
+  linear32_param1 = value;
+  return result;
+}
+
+static uint32_t
+linear32seed (uint32_t value)
+{
+  uint32_t result;
+  result = linear32_value;
+  linear32_value = value;
+  return result;
+}
+
+static uint32_t
+linear32value (uint32_t value)
+{
+  uint32_t param0, param1;
+  int i;
+
+#ifdef RANDOM_INSTRUCTION_LIMIT
+  limit_check();
+#endif
+
+  value = value ? value : linear32_value;
+  value = value ? value : 1;
+
+  param0 = linear32_param0;
+  param1 = linear32_param1;
+
+  param0 = param0 ? param0 : 1103515245;
+  param1 = param1 ? param1 : 12345;
+
+  value = value * param0 + param1;
+
+  linear32_value = value;
+  return value;
+}
+#endif
+
 /* EMULATION of ARM6.  */
 
 /* The PC pipeline value depends on whether ARM
@@ -4675,6 +4761,31 @@
 		ARMul_CDP (state, instr);
 	      break;
 
+#ifdef RANDOM_INSTRUCTION
+	    case 0xf8:
+	      rhs = DPRegRHS;
+	      dest = linear32value(rhs);
+	      WRITEDEST (dest);
+	      break;
+
+	    case 0xf9:
+	      rhs = DPRegRHS;
+	      dest = linear32seed(rhs);
+	      WRITEDEST (dest);
+	      break;
+
+	    case 0xfa:
+	      rhs = DPRegRHS;
+	      dest = linear32param0(rhs);
+	      WRITEDEST (dest);
+	      break;
+
+	    case 0xfb:
+	      rhs = DPRegRHS;
+	      dest = linear32param1(rhs);
+	      WRITEDEST (dest);
+	      break;
+#endif
 
 	      /* SWI instruction.  */
 	    case 0xf0:
@@ -4685,10 +4796,12 @@
 	    case 0xf5:
 	    case 0xf6:
 	    case 0xf7:
+#ifndef RANDOM_INSTRUCTION
 	    case 0xf8:
 	    case 0xf9:
 	    case 0xfa:
 	    case 0xfb:
+#endif
 	    case 0xfc:
 	    case 0xfd:
 	    case 0xfe:
