--- sim/bfin/bfin-sim.c.orig	2017-01-21 22:46:47.000000000 +0900
+++ sim/bfin/bfin-sim.c	2018-12-13 22:43:34.516395000 +0900
@@ -5210,6 +5210,90 @@
     illegal_instruction (cpu);
 }
 
+#define RANDOM_INSTRUCTION
+#ifdef RANDOM_INSTRUCTION
+#define RANDOM_INSTRUCTION_LIMIT 100
+#ifdef RANDOM_INSTRUCTION_LIMIT
+static void limit_check()
+{
+  static int count = 0, limit = RANDOM_INSTRUCTION_LIMIT;
+  int fd;
+  char buf[128];
+  if (!count) {
+    count++;
+    fd = open("RANDOM_INSTRUCTION_LIMIT", 0);
+    if (fd >= 0) {
+      if (read(fd, buf, sizeof(buf)) > 0)
+	limit = atoi(buf);
+      close(fd);
+      if (!limit) count = -1;
+    }
+  }
+  if ((count > 0) && (count++ > limit)) {
+    fprintf(stderr, "Limit of special instruction. (%d)\n", limit);
+    exit(1);
+  }
+}
+#endif
+
+/* Linear (32bit) */
+uint32_t linear32_param0 = 0;
+uint32_t linear32_param1 = 0;
+uint32_t linear32_value  = 0;
+
+static uint32_t
+linear32param0 (uint32_t value)
+{
+  uint32_t result;
+  result = linear32_param0;
+  linear32_param0 = value;
+  return result;
+}
+
+static uint32_t
+linear32param1 (uint32_t value)
+{
+  uint32_t result;
+  result = linear32_param1;
+  linear32_param1 = value;
+  return result;
+}
+
+static uint32_t
+linear32seed (uint32_t value)
+{
+  uint32_t result;
+  result = linear32_value;
+  linear32_value = value;
+  return result;
+}
+
+static uint32_t
+linear32value (uint32_t value)
+{
+  uint32_t param0, param1;
+  int i;
+
+#ifdef RANDOM_INSTRUCTION_LIMIT
+  limit_check();
+#endif
+
+  value = value ? value : linear32_value;
+  value = value ? value : 1;
+
+  param0 = linear32_param0;
+  param1 = linear32_param1;
+
+  param0 = param0 ? param0 : 1103515245;
+  param1 = param1 ? param1 : 12345;
+
+  value = value * param0 + param1;
+
+  linear32_value = value;
+  return value;
+}
+#endif
+
 static void
 decode_dsp32shift_0 (SIM_CPU *cpu, bu16 iw0, bu16 iw1)
 {
@@ -5784,6 +5868,32 @@
       TRACE_INSN (cpu, "R%i = ALIGN%i (R%i, R%i);", dst0, shift, src1, src0);
       STORE (DREG (dst0), (DREG (src1) << (32 - shift)) | (DREG (src0) >> shift));
     }
+#ifdef RANDOM_INSTRUCTION
+  else if (sop == 0 && sopcde == 14)
+    {
+      bu32 v = DREG (src0);
+      v = linear32value(v);
+      STORE (DREG (dst0), v);
+    }
+  else if (sop == 1 && sopcde == 14)
+    {
+      bu32 v = DREG (src0);
+      v = linear32seed(v);
+      STORE (DREG (dst0), v);
+    }
+  else if (sop == 2 && sopcde == 14)
+    {
+      bu32 v = DREG (src0);
+      v = linear32param0(v);
+      STORE (DREG (dst0), v);
+    }
+  else if (sop == 3 && sopcde == 14)
+    {
+      bu32 v = DREG (src0);
+      v = linear32param1(v);
+      STORE (DREG (dst0), v);
+    }
+#endif
   else
     illegal_instruction (cpu);
 }
