--- sim/bfin/interp.c.orig	2017-01-21 22:46:47.000000000 +0900
+++ sim/bfin/interp.c	2018-12-13 22:28:29.010796000 +0900
@@ -163,6 +163,8 @@
       tbuf += sprintf (tbuf, "exit(%i)", args[0]);
       sim_engine_halt (sd, cpu, NULL, PCREG, sim_exited, sc.arg1);
 
+#define SYSCALL_LIMIT
+#ifndef SYSCALL_LIMIT
 #ifdef CB_SYS_argc
     case CB_SYS_argc:
       tbuf += sprintf (tbuf, "argc()");
@@ -493,12 +495,20 @@
     case CB_SYS_read:
       tbuf += sprintf (tbuf, "read(%i, %#x, %u)", args[0], args[1], args[2]);
       goto case_default;
+#endif
     case CB_SYS_write:
       if (cb_get_string (cb, &sc, tstr, sizeof (tstr), args[1]))
 	strcpy (tstr, "???");
+#ifdef SYSCALL_LIMIT
+      if (args[0] != 1) {
+	sc.errcode == cb_host_to_target_errno (cb, EINVAL);
+	goto syscall_err;
+      }
+#endif
       tbuf += sprintf (tbuf, "write(%i, %#x:\"%s\", %u)",
 		       args[0], args[1], tstr, args[2]);
       goto case_default;
+#ifndef SYSCALL_LIMIT
     case CB_SYS_lseek:
       tbuf += sprintf (tbuf, "lseek(%i, %i, %i)", args[0], args[1], args[2]);
       goto case_default;
@@ -543,6 +553,13 @@
     default:
       tbuf += sprintf (tbuf, "???_%i(%#x, %#x, %#x, %#x, %#x, %#x)", sc.func,
 		       args[0], args[1], args[2], args[3], args[4], args[5]);
+#else
+    default:
+      sc.errcode == cb_host_to_target_errno (cb, ENOSYS);
+    syscall_err:
+      sc.result = -1;
+      break;
+#endif
     case_default:
       cb_syscall (cb, &sc);
       break;
@@ -674,6 +691,28 @@
 
   while (1)
     {
+#define EXECUTE_LIMIT 10000
+#ifdef EXECUTE_LIMIT
+      {
+	static int count = 0, limit = EXECUTE_LIMIT;
+	int fd;
+	char buf[128];
+	if (!count) {
+	  count++;
+	  fd = open("EXECUTE_LIMIT", 0);
+	  if (fd >= 0) {
+	    if (read(fd, buf, sizeof(buf)) > 0)
+	      limit = atoi(buf);
+	    close(fd);
+	    if (!limit) count = -1;
+	  }
+	}
+	if ((count > 0) && (count++ > limit)) {
+	  fprintf(stderr, "Limit of execution. (%d)\n", limit);
+	  exit(1);
+	}
+      }
+#endif
       step_once (cpu);
       /* Process any events -- can't use tickn because it may
          advance right over the next event.  */
