--- sim/microblaze/interp.c.orig	2017-01-21 22:46:47.000000000 +0900
+++ sim/microblaze/interp.c	2018-12-13 22:44:19.044517000 +0900
@@ -107,6 +107,97 @@
   CPU.imm_enable = 0;
 }
 
+#define RANDOM_INSTRUCTION
+#ifdef RANDOM_INSTRUCTION
+#define RANDOM_INSTRUCTION_LIMIT 100
+#ifdef RANDOM_INSTRUCTION_LIMIT
+static void limit_check()
+{
+  static int count = 0, limit = RANDOM_INSTRUCTION_LIMIT;
+  int fd;
+  char buf[128];
+  if (!count) {
+    count++;
+    fd = open("RANDOM_INSTRUCTION_LIMIT", 0);
+    if (fd >= 0) {
+      if (read(fd, buf, sizeof(buf)) > 0)
+	limit = atoi(buf);
+      close(fd);
+      if (!limit) count = -1;
+    }
+  }
+  if ((count > 0) && (count++ > limit)) {
+    fprintf(stderr, "Limit of special instruction. (%d)\n", limit);
+    exit(1);
+  }
+}
+#endif
+
+typedef enum {
+  XORSHIFT_SEQ_LRL = 0,
+  XORSHIFT_SEQ_RLR = 1,
+  XORSHIFT_SEQ_LLR = 2,
+  XORSHIFT_SEQ_RRL = 3
+} xorshift_seq_t;
+
+/* xorshift (32bit) */
+uint32_t xorshift32_param = 0;
+uint32_t xorshift32_value = 0;
+
+static uint32_t
+xorshift32param (uint32_t value)
+{
+  uint32_t result;
+  result = xorshift32_param;
+  xorshift32_param = (value & ((1 << 17) - 1));
+  return result;
+}
+
+static uint32_t
+xorshift32seed (uint32_t value)
+{
+  uint32_t result;
+  result = xorshift32_value;
+  xorshift32_value = value;
+  return result;
+}
+
+static uint32_t
+xorshift32value (uint32_t value)
+{
+  uint32_t v = value;
+  uint32_t result = 0;
+  int seq, a, b, c;
+
+#ifdef RANDOM_INSTRUCTION_LIMIT
+  limit_check();
+#endif
+
+  seq = xorshift32_param & 3;
+
+  a = (xorshift32_param >>  2) & 0x1f;
+  b = (xorshift32_param >>  7) & 0x1f;
+  c = (xorshift32_param >> 12) & 0x1f;
+  a = a ? a :  9;
+  b = b ? b :  5;
+  c = c ? c : 14;
+
+  v = v ? v : xorshift32_value;
+  v = v ? v : 0x900dface;
+
+  switch (seq) {
+  case XORSHIFT_SEQ_LRL: v = v ^ (v<<a); v = v ^ (v>>b); v = v ^ (v<<c); break;
+  case XORSHIFT_SEQ_RLR: v = v ^ (v>>a); v = v ^ (v<<b); v = v ^ (v>>c); break;
+  case XORSHIFT_SEQ_LLR: v = v ^ (v<<a); v = v ^ (v<<c); v = v ^ (v>>b); break;
+  case XORSHIFT_SEQ_RRL: v = v ^ (v>>a); v = v ^ (v>>c); v = v ^ (v<<b); break;
+  default: break;
+  }
+  xorshift32_value = v;
+  result = v;
+  return result;
+}
+#endif
+
 static int tracing = 0;
 
 void
@@ -162,7 +253,56 @@
       delay_slot_enable = 0;
       branch_taken = 0;
       if (op == microblaze_brk)
+#ifdef RANDOM_INSTRUCTION
+        {
+	  int sysno;
+	  int arg0, arg1, arg2;
+	  char buf[256];
+	  int i, len;
+	  sysno = MEM_RD_WORD((RB & ~0x3) +  0);
+	  arg0  = MEM_RD_WORD((RB & ~0x3) +  4);
+	  arg1  = MEM_RD_WORD((RB & ~0x3) +  8);
+	  arg2  = MEM_RD_WORD((RB & ~0x3) + 12);
+	  switch (sysno) {
+	  case 3: /* read */
+	    len = (arg2 < sizeof(buf)) ? arg2 : sizeof(buf);
+	    if (len < 0) {
+	      RD = -1;
+	      break;
+	    }
+	    len = read(arg0, buf, len);
+	    if (len > 0) {
+	      for (i = 0; i < len; i++) {
+		MEM_WR_BYTE(arg1 + i, buf[i]);
+	      }
+	    }
+	    RD = len;
+	    break;
+	  case 4: /* write */
+	    len = (arg2 < sizeof(buf)) ? arg2 : sizeof(buf);
+	    if (len < 0) {
+	      RD = -1;
+	      break;
+	    }
+	    if (len > 0) {
+	      for (i = 0; i < len; i++) {
+		buf[i] = MEM_RD_UBYTE(arg1 + i);
+	      }
+	    }
+	    len = write(arg0, buf, len);
+	    RD = len;
+	    break;
+	  default:
+	    RD = -1;
+	    break;
+	  }
+	  PC += INST_SIZE;
+	  insts += 1;
+	  bonus_cycles++;
+      }
+#else
 	sim_engine_halt (sd, NULL, NULL, NULL_CIA, sim_stopped, SIM_SIGTRAP);
+#endif
       else if (inst == MICROBLAZE_HALT_INST)
 	{
 	  insts += 1;
