--- sim/mips/interp.c.orig	2016-08-02 00:50:21.000000000 +0900
+++ sim/mips/interp.c	2018-12-13 22:44:02.018507000 +0900
@@ -1181,6 +1181,8 @@
   switch (reason)
     {
 
+#define SYSCALL_LIMIT
+#ifndef SYSCALL_LIMIT
     case 6: /* int open(char *path,int flags) */
       {
 	char *path = fetch_str (sd, A0);
@@ -1199,12 +1201,20 @@
 	free (buf);
       }
       break;
+#endif
 
     case 8: /* int write(int file,char *ptr,int len) */
       {
 	int fd = A0;
 	int nr = A2;
 	char *buf = zalloc (nr);
+#ifdef SYSCALL_LIMIT
+	if (fd != 1) {
+	  V0 = -1;
+	  free (buf);
+	  break;
+	}
+#endif
 	sim_read (sd, A1, (unsigned char *)buf, nr);
 	V0 = sim_io_write (sd, fd, buf, nr);
 	if (fd == 1)
@@ -1215,6 +1225,7 @@
 	break;
       }
 
+#ifndef SYSCALL_LIMIT
     case 10: /* int close(int file) */
       {
 	V0 = sim_io_close (sd, (int)A0);
@@ -1250,6 +1261,7 @@
         sim_io_write_stdout (sd, &tmp, sizeof(char));
 	break;
       }
+#endif
 
     case 17: /* void _exit() */
       {
@@ -1259,6 +1271,7 @@
 	break;
       }
 
+#ifndef SYSCALL_LIMIT
     case 28: /* PMON flush_cache */
       break;
 
@@ -1423,6 +1436,7 @@
 	  }
 	break;
       }
+#endif
 
     default:
       /* Unknown reason.  */
@@ -2489,5 +2503,154 @@
 }
 
 
+#define RANDOM_INSTRUCTION
+#ifdef RANDOM_INSTRUCTION
+#define RANDOM_INSTRUCTION_LIMIT 100
+#ifdef RANDOM_INSTRUCTION_LIMIT
+static void limit_check()
+{
+  static int count = 0, limit = RANDOM_INSTRUCTION_LIMIT;
+  int fd;
+  char buf[128];
+  if (!count) {
+    count++;
+    fd = open("RANDOM_INSTRUCTION_LIMIT", 0);
+    if (fd >= 0) {
+      if (read(fd, buf, sizeof(buf)) > 0)
+	limit = atoi(buf);
+      close(fd);
+      if (!limit) count = -1;
+    }
+  }
+  if ((count > 0) && (count++ > limit)) {
+    fprintf(stderr, "Limit of special instruction. (%d)\n", limit);
+    exit(1);
+  }
+}
+#endif
+
+typedef enum {
+  XORSHIFT_SEQ_LRL = 0,
+  XORSHIFT_SEQ_RLR = 1,
+  XORSHIFT_SEQ_LLR = 2,
+  XORSHIFT_SEQ_RRL = 3
+} xorshift_seq_t;
+
+/* xorshift (32bit) */
+uint32_t xorshift32_param = 0;
+uint32_t xorshift32_value = 0;
+
+uint32_t
+xorshift32param (uint32_t value)
+{
+  uint32_t result;
+  result = xorshift32_param;
+  xorshift32_param = (value & ((1 << 17) - 1));
+  return result;
+}
+
+uint32_t
+xorshift32seed (uint32_t value)
+{
+  uint32_t result;
+  result = xorshift32_value;
+  xorshift32_value = value;
+  return result;
+}
+
+uint32_t
+xorshift32value (uint32_t value)
+{
+  uint32_t v = value;
+  uint32_t result = 0;
+  int seq, a, b, c;
+
+#ifdef RANDOM_INSTRUCTION_LIMIT
+  limit_check();
+#endif
+
+  seq = xorshift32_param & 3;
+
+  a = (xorshift32_param >>  2) & 0x1f;
+  b = (xorshift32_param >>  7) & 0x1f;
+  c = (xorshift32_param >> 12) & 0x1f;
+  a = a ? a :  1;
+  b = b ? b :  3;
+  c = c ? c : 10;
+
+  v = v ? v : xorshift32_value;
+  v = v ? v : 0xbadbeef;
+
+  switch (seq) {
+  case XORSHIFT_SEQ_LRL: v = v ^ (v<<a); v = v ^ (v>>b); v = v ^ (v<<c); break;
+  case XORSHIFT_SEQ_RLR: v = v ^ (v>>a); v = v ^ (v<<b); v = v ^ (v>>c); break;
+  case XORSHIFT_SEQ_LLR: v = v ^ (v<<a); v = v ^ (v<<c); v = v ^ (v>>b); break;
+  case XORSHIFT_SEQ_RRL: v = v ^ (v>>a); v = v ^ (v>>c); v = v ^ (v<<b); break;
+  default: break;
+  }
+  xorshift32_value = v;
+  result = v;
+  return result;
+}
+
+/* xorshift (64bit) */
+uint64_t xorshift64_param = 0;
+uint64_t xorshift64_value = 0;
+
+uint64_t
+xorshift64param (uint64_t value)
+{
+  uint64_t result;
+  result = xorshift64_param;
+  xorshift64_param = (value & ((1 << 20) - 1));
+  return result;
+}
+
+uint64_t
+xorshift64seed (uint64_t value)
+{
+  uint64_t result;
+  result = xorshift64_value;
+  xorshift64_value = value;
+  return result;
+}
+
+uint64_t
+xorshift64value (uint64_t value)
+{
+  uint64_t v = value;
+  uint64_t result = 0;
+  int seq, a, b, c;
+
+#ifdef RANDOM_INSTRUCTION_LIMIT
+  limit_check();
+#endif
+
+  seq = xorshift64_param & 3;
+
+  a = (xorshift64_param >>  2) & 0x3f;
+  b = (xorshift64_param >>  8) & 0x3f;
+  c = (xorshift64_param >> 14) & 0x3f;
+  a = a ? a : 34;
+  b = b ? b : 41;
+  c = c ? c : 23;
+
+  v = v ? v : xorshift64_value;
+  v = v ? v : 0xbadbadbadbadbeef;
+
+  switch (seq) {
+  case XORSHIFT_SEQ_LRL: v = v ^ (v<<a); v = v ^ (v>>b); v = v ^ (v<<c); break;
+  case XORSHIFT_SEQ_RLR: v = v ^ (v>>a); v = v ^ (v<<b); v = v ^ (v>>c); break;
+  case XORSHIFT_SEQ_LLR: v = v ^ (v<<a); v = v ^ (v<<c); v = v ^ (v>>b); break;
+  case XORSHIFT_SEQ_RRL: v = v ^ (v>>a); v = v ^ (v>>c); v = v ^ (v<<b); break;
+  default: break;
+  }
+  xorshift64_value = v;
+  result = v;
+  return result;
+}
+#endif
+
+
 /*---------------------------------------------------------------------------*/
 /*> EOF interp.c <*/
