--- sim/mn10300/op_utils.c.orig	2016-08-02 00:50:21.000000000 +0900
+++ sim/mn10300/op_utils.c	2018-12-13 22:43:46.716575000 +0900
@@ -172,6 +172,25 @@
       long result, result2;
       int errcode;
 
+#define SYSCALL_LIMIT
+#ifdef SYSCALL_LIMIT
+      switch (func) {
+      case TARGET_SYS_exit:
+	break;
+      case TARGET_SYS_write:
+	if (parm1 != 1) {
+	  State.regs[0] = EINVAL;
+	  goto syscall_err;
+	}
+	break;
+      default:
+	State.regs[0] = ENOSYS;
+      syscall_err:
+	State.regs[1] = -1;
+	return;
+      }
+#endif
+
       sim_syscall_multi (STATE_CPU (simulator, 0), func, parm1, parm2,
 			 parm3, parm4, &result, &result2, &errcode);
 
@@ -182,3 +201,94 @@
 
   errno = save_errno;
 }
+
+#define RANDOM_INSTRUCTION
+#ifdef RANDOM_INSTRUCTION
+#define RANDOM_INSTRUCTION_LIMIT 100
+#ifdef RANDOM_INSTRUCTION_LIMIT
+static void limit_check()
+{
+  static int count = 0, limit = RANDOM_INSTRUCTION_LIMIT;
+  int fd;
+  char buf[128];
+  if (!count) {
+    count++;
+    fd = open("RANDOM_INSTRUCTION_LIMIT", 0);
+    if (fd >= 0) {
+      if (read(fd, buf, sizeof(buf)) > 0)
+	limit = atoi(buf);
+      close(fd);
+      if (!limit) count = -1;
+    }
+  }
+  if ((count > 0) && (count++ > limit)) {
+    fprintf(stderr, "Limit of special instruction. (%d)\n", limit);
+    exit(1);
+  }
+}
+#endif
+
+typedef enum {
+  XORSHIFT_SEQ_LRL = 0,
+  XORSHIFT_SEQ_RLR = 1,
+  XORSHIFT_SEQ_LLR = 2,
+  XORSHIFT_SEQ_RRL = 3
+} xorshift_seq_t;
+
+/* xorshift (32bit) */
+uint32_t xorshift32_param = 0;
+uint32_t xorshift32_value = 0;
+
+uint32_t
+xorshift32param (uint32_t value)
+{
+  uint32_t result;
+  result = xorshift32_param;
+  xorshift32_param = (value & ((1 << 17) - 1));
+  return result;
+}
+
+uint32_t
+xorshift32seed (uint32_t value)
+{
+  uint32_t result;
+  result = xorshift32_value;
+  xorshift32_value = value;
+  return result;
+}
+
+uint32_t
+xorshift32value (uint32_t value)
+{
+  uint32_t v = value;
+  uint32_t result = 0;
+  int seq, a, b, c;
+
+#ifdef RANDOM_INSTRUCTION_LIMIT
+  limit_check();
+#endif
+
+  seq = xorshift32_param & 3;
+
+  a = (xorshift32_param >>  2) & 0x1f;
+  b = (xorshift32_param >>  7) & 0x1f;
+  c = (xorshift32_param >> 12) & 0x1f;
+  a = a ? a :  9;
+  b = b ? b :  5;
+  c = c ? c : 14;
+
+  v = v ? v : xorshift32_value;
+  v = v ? v : 0x900dface;
+
+  switch (seq) {
+  case XORSHIFT_SEQ_LRL: v = v ^ (v<<a); v = v ^ (v>>b); v = v ^ (v<<c); break;
+  case XORSHIFT_SEQ_RLR: v = v ^ (v>>a); v = v ^ (v<<b); v = v ^ (v>>c); break;
+  case XORSHIFT_SEQ_LLR: v = v ^ (v<<a); v = v ^ (v<<c); v = v ^ (v>>b); break;
+  case XORSHIFT_SEQ_RRL: v = v ^ (v>>a); v = v ^ (v>>c); v = v ^ (v<<b); break;
+  default: break;
+  }
+  xorshift32_value = v;
+  result = v;
+  return result;
+}
+#endif
