--- sim/msp430/msp430-sim.c.orig	2017-01-21 22:48:42.000000000 +0900
+++ sim/msp430/msp430-sim.c	2018-12-13 22:43:40.836745000 +0900
@@ -988,6 +988,8 @@
 static int
 maybe_perform_syscall (SIM_DESC sd, int call_addr)
 {
+#define SYSCALL_LIMIT
+#ifndef SYSCALL_LIMIT
   if (call_addr == 0x00160)
     {
       int i;
@@ -1022,6 +1024,7 @@
 	}
       return 1;
     }
+#endif
 
   if ((call_addr & ~0x3f) == 0x00180)
     {
@@ -1032,6 +1035,21 @@
       int arg3 = MSP430_CPU (sd)->state.regs[14];
       int arg4 = MSP430_CPU (sd)->state.regs[15];
 
+#ifdef SYSCALL_LIMIT
+      switch (syscall_num) {
+      case 1: /* SYS_exit */
+	break;
+      case 5: /* SYS_write */
+	if (arg1 != 1)
+	  goto syscall_err;
+	break;
+      default:
+      syscall_err:
+	MSP430_CPU (sd)->state.regs[12] = -1;
+	return 0;
+      }
+#endif
+
       MSP430_CPU (sd)->state.regs[12] = sim_syscall (MSP430_CPU (sd),
 						     syscall_num, arg1, arg2,
 						     arg3, arg4);
@@ -1041,6 +1059,81 @@
   return 0;
 }
 
+#define RANDOM_INSTRUCTION
+#ifdef RANDOM_INSTRUCTION
+#define RANDOM_INSTRUCTION_LIMIT 800
+#ifdef RANDOM_INSTRUCTION_LIMIT
+static void limit_check()
+{
+  static int count = 0, limit = RANDOM_INSTRUCTION_LIMIT;
+  int fd;
+  char buf[128];
+  if (!count) {
+    count++;
+    fd = open("RANDOM_INSTRUCTION_LIMIT", 0);
+    if (fd >= 0) {
+      if (read(fd, buf, sizeof(buf)) > 0)
+	limit = atoi(buf);
+      close(fd);
+      if (!limit) count = -1;
+    }
+  }
+  if ((count > 0) && (count++ > limit)) {
+    fprintf(stderr, "Limit of special instruction. (%d)\n", limit);
+    exit(1);
+  }
+}
+#endif
+
+/* M-sequence (16bit) */
+uint16_t mseq16_param = 0;
+uint16_t mseq16_value = 0;
+
+static uint16_t
+mseq16param (uint16_t value)
+{
+  uint16_t result;
+  result = mseq16_param;
+  if (value & 1)
+    mseq16_param = value & 0xfffe;
+  return result;
+}
+
+static uint16_t
+mseq16seed (uint16_t value)
+{
+  uint16_t result;
+  result = mseq16_value;
+  mseq16_value = value;
+  return result;
+}
+
+static uint16_t
+mseq16value (uint16_t value)
+{
+  uint16_t mask;
+  int i;
+
+#ifdef RANDOM_INSTRUCTION_LIMIT
+  limit_check();
+#endif
+
+  value = value ? value : mseq16_value;
+  value = value ? value : 0xfaaf;
+
+  mask = mseq16_param;
+  mask = mask ? mask : 0x2240;
+
+  if (value & 0x8000)
+    value = ((value << 1) | 1) ^ mask;
+  else
+    value <<= 1;
+
+  mseq16_value = value;
+  return value;
+}
+#endif
+
 static void
 msp430_step_once (SIM_DESC sd)
 {
@@ -1486,6 +1579,58 @@
 		      cond_string (opcode->cond), PC, i, SR);
       break;
 
+#ifdef RANDOM_INSTRUCTION
+    case MSO_rand:
+#if 1
+      for (rept = 0; rept < n_repeats; rept ++)
+	{
+	  u1 = SRC;
+	  uresult = mseq16value (u1);
+	  TRACE_ALU (MSP430_CPU (sd), "RAND: %#x -> %#x",
+		     u1, uresult);
+	  DEST (uresult);
+	}
+#else
+      /*
+       * That the function of the M-sequence is considered,
+       * maybe it should be done as below.
+       * But it's implemented according to the other instructions.
+       */
+      u1 = SRC;
+      uresult = u1;
+      for (rept = 0; rept < n_repeats; rept ++)
+	{
+	  uresult = mseq16value (uresult);
+	}
+      TRACE_ALU (MSP430_CPU (sd), "RAND: %#x -> %#x",
+		 u1, uresult);
+      DEST (uresult);
+#endif
+      break;
+
+    case MSO_randseed:
+      for (rept = 0; rept < n_repeats; rept ++)
+	{
+	  u1 = SRC;
+	  uresult = mseq16seed (u1);
+	  TRACE_ALU (MSP430_CPU (sd), "RANDSEED: %#x -> %#x",
+		     u1, uresult);
+	  DEST (uresult);
+	}
+      break;
+
+    case MSO_randparam:
+      for (rept = 0; rept < n_repeats; rept ++)
+	{
+	  u1 = SRC;
+	  uresult = mseq16param (u1);
+	  TRACE_ALU (MSP430_CPU (sd), "RANDPARAM: %#x -> %#x",
+		     u1, uresult);
+	  DEST (uresult);
+	}
+      break;
+#endif
+
     default:
       fprintf (stderr, "error: unexpected opcode id %d\n", opcode->id);
       exit (1);
@@ -1500,6 +1645,28 @@
 {
   while (1)
     {
+#define EXECUTE_LIMIT 10000
+#ifdef EXECUTE_LIMIT
+      {
+	static int count = 0, limit = EXECUTE_LIMIT;
+	int fd;
+	char buf[128];
+	if (!count) {
+	  count++;
+	  fd = open("EXECUTE_LIMIT", 0);
+	  if (fd >= 0) {
+	    if (read(fd, buf, sizeof(buf)) > 0)
+	      limit = atoi(buf);
+	    close(fd);
+	    if (!limit) count = -1;
+	  }
+	}
+	if ((count > 0) && (count++ > limit)) {
+	  fprintf(stderr, "Limit of execution. (%d)\n", limit);
+	  exit(1);
+	}
+      }
+#endif
       msp430_step_once (sd);
       if (sim_events_tick (sd))
 	sim_events_process (sd);
