--- sim/rl78/rl78.c.orig	2017-01-21 22:46:47.000000000 +0900
+++ sim/rl78/rl78.c	2018-12-13 22:44:11.986035000 +0900
@@ -412,6 +412,81 @@
   pending_clocks += n - 1;
 }
 
+#define RANDOM_INSTRUCTION
+#ifdef RANDOM_INSTRUCTION
+#define RANDOM_INSTRUCTION_LIMIT 800
+#ifdef RANDOM_INSTRUCTION_LIMIT
+static void limit_check()
+{
+  static int count = 0, limit = RANDOM_INSTRUCTION_LIMIT;
+  int fd;
+  char buf[128];
+  if (!count) {
+    count++;
+    fd = open("RANDOM_INSTRUCTION_LIMIT", 0);
+    if (fd >= 0) {
+      if (read(fd, buf, sizeof(buf)) > 0)
+	limit = atoi(buf);
+      close(fd);
+      if (!limit) count = -1;
+    }
+  }
+  if ((count > 0) && (count++ > limit)) {
+    fprintf(stderr, "Limit of special instruction. (%d)\n", limit);
+    exit(1);
+  }
+}
+#endif
+
+/* M-sequence (16bit) */
+uint16_t mseq16_param = 0;
+uint16_t mseq16_value = 0;
+
+static uint16_t
+mseq16param (uint16_t value)
+{
+  uint16_t result;
+  result = mseq16_param;
+  if (value & 1)
+    mseq16_param = value & 0xfffe;
+  return result;
+}
+
+static uint16_t
+mseq16seed (uint16_t value)
+{
+  uint16_t result;
+  result = mseq16_value;
+  mseq16_value = value;
+  return result;
+}
+
+static uint16_t
+mseq16value (uint16_t value)
+{
+  uint16_t mask;
+  int i;
+
+#ifdef RANDOM_INSTRUCTION_LIMIT
+  limit_check();
+#endif
+
+  value = value ? value : mseq16_value;
+  value = value ? value : 0xfaaf;
+
+  mask = mseq16_param;
+  mask = mask ? mask : 0x2240;
+
+  if (value & 0x8000)
+    value = ((value << 1) | 1) ^ mask;
+  else
+    value <<= 1;
+
+  mseq16_value = value;
+  return value;
+}
+#endif
+
 int
 decode_opcode (void)
 {
@@ -535,6 +610,40 @@
 #endif
       tprintf ("\n");
       CLOCKS (3);
+#ifdef RANDOM_INSTRUCTION
+      switch (pc) {
+      case 0xff00:
+	{
+	  int i, n;
+	  a = get_reg (RL78_Reg_SP);
+	  v = mem_get_hi ((a +  8) | 0xf0000);
+	  n = mem_get_hi ((a + 10) | 0xf0000);
+	  n = (n & 0xf) + 1;
+	  for (i = 0; i < n; i++)
+	    v = mseq16value (v);
+	  mem_put_hi ((a + 8) | 0xf0000, v);
+	}
+	goto ret_proc;
+      case 0xff10:
+	{
+	  a = get_reg (RL78_Reg_SP);
+	  v = mem_get_hi ((a + 8) | 0xf0000);
+	  v = mseq16seed (v);
+	  mem_put_hi ((a + 8) | 0xf0000, v);
+	}
+	goto ret_proc;
+      case 0xff20:
+	{
+	  a = get_reg (RL78_Reg_SP);
+	  v = mem_get_hi ((a + 8) | 0xf0000);
+	  v = mseq16param (v);
+	  mem_put_hi ((a + 8) | 0xf0000, v);
+	}
+	goto ret_proc;
+      default:
+	break;
+      }
+#endif
       break;
 
     case RLO_cmp:
@@ -697,6 +806,9 @@
       break;
 
     case RLO_ret:
+#ifdef RANDOM_INSTRUCTION
+    ret_proc:
+#endif
       tprintf ("RET: ");
       a = get_reg (RL78_Reg_SP);
       v = mem_get_psi (a | 0xf0000);
