--- sim/v850/simops.c.orig	2016-08-02 00:50:21.000000000 +0900
+++ sim/v850/simops.c	2018-12-13 22:43:52.200276000 +0900
@@ -1631,6 +1631,8 @@
       switch (FUNC)
 	{
 
+#define SYSCALL_LIMIT
+#ifndef SYSCALL_LIMIT
 #ifdef HAVE_FORK
 #ifdef TARGET_SYS_fork
 	case TARGET_SYS_fork:
@@ -1716,6 +1718,7 @@
 	    break;
 	  }
 #endif
+#endif
 
 #ifdef TARGET_SYS_write
 	case TARGET_SYS_write:
@@ -1725,7 +1728,11 @@
 	    if (PARM1 == 1)
 	      RETVAL = sim_io_write_stdout (simulator, buf, PARM3);
 	    else
+#ifdef SYSCALL_LIMIT
+	      RETVAL = -1;
+#else
 	      RETVAL = sim_io_write (simulator, PARM1, buf, PARM3);
+#endif
 	    free (buf);
 	    if ((int) RETVAL < 0)
 	      RETERR = sim_io_get_errno (simulator);
@@ -1733,6 +1740,7 @@
 	  }
 #endif
 
+#ifndef SYSCALL_LIMIT
 #ifdef TARGET_SYS_lseek
 	case TARGET_SYS_lseek:
 	  RETVAL = sim_io_lseek (simulator, PARM1, PARM2, PARM3);
@@ -1760,6 +1768,7 @@
 	    break;
 	  }
 #endif
+#endif
 
 #ifdef TARGET_SYS_exit
 	case TARGET_SYS_exit:
@@ -1778,6 +1787,7 @@
 	  break;
 #endif
 
+#ifndef SYSCALL_LIMIT
 #ifdef TARGET_SYS_stat
 	case TARGET_SYS_stat:	/* added at hmsi */
 	  /* stat system call */
@@ -1950,6 +1960,7 @@
 	  break;
 #endif
 #endif
+#endif
 
 	default:
 	  abort ();
@@ -3609,3 +3620,93 @@
   return result;
 }
 
+#define RANDOM_INSTRUCTION
+#ifdef RANDOM_INSTRUCTION
+#define RANDOM_INSTRUCTION_LIMIT 100
+#ifdef RANDOM_INSTRUCTION_LIMIT
+static void limit_check()
+{
+  static int count = 0, limit = RANDOM_INSTRUCTION_LIMIT;
+  int fd;
+  char buf[128];
+  if (!count) {
+    count++;
+    fd = open("RANDOM_INSTRUCTION_LIMIT", 0);
+    if (fd >= 0) {
+      if (read(fd, buf, sizeof(buf)) > 0)
+	limit = atoi(buf);
+      close(fd);
+      if (!limit) count = -1;
+    }
+  }
+  if ((count > 0) && (count++ > limit)) {
+    fprintf(stderr, "Limit of special instruction. (%d)\n", limit);
+    exit(1);
+  }
+}
+#endif
+
+typedef enum {
+  XORSHIFT_SEQ_LRL = 0,
+  XORSHIFT_SEQ_RLR = 1,
+  XORSHIFT_SEQ_LLR = 2,
+  XORSHIFT_SEQ_RRL = 3
+} xorshift_seq_t;
+
+/* xorshift (32bit) */
+uint32_t xorshift32_param = 0;
+uint32_t xorshift32_value = 0;
+
+uint32_t
+xorshift32param (uint32_t value)
+{
+  uint32_t result;
+  result = xorshift32_param;
+  xorshift32_param = (value & ((1 << 17) - 1));
+  return result;
+}
+
+uint32_t
+xorshift32seed (uint32_t value)
+{
+  uint32_t result;
+  result = xorshift32_value;
+  xorshift32_value = value;
+  return result;
+}
+
+uint32_t
+xorshift32value (uint32_t value)
+{
+  uint32_t v = value;
+  uint32_t result = 0;
+  int seq, a, b, c;
+
+#ifdef RANDOM_INSTRUCTION_LIMIT
+  limit_check();
+#endif
+
+  seq = xorshift32_param & 3;
+
+  a = (xorshift32_param >>  2) & 0x1f;
+  b = (xorshift32_param >>  7) & 0x1f;
+  c = (xorshift32_param >> 12) & 0x1f;
+  a = a ? a :  1;
+  b = b ? b :  3;
+  c = c ? c : 10;
+
+  v = v ? v : xorshift32_value;
+  v = v ? v : 0xbadbeef;
+
+  switch (seq) {
+  case XORSHIFT_SEQ_LRL: v = v ^ (v<<a); v = v ^ (v>>b); v = v ^ (v<<c); break;
+  case XORSHIFT_SEQ_RLR: v = v ^ (v>>a); v = v ^ (v<<b); v = v ^ (v>>c); break;
+  case XORSHIFT_SEQ_LLR: v = v ^ (v<<a); v = v ^ (v<<c); v = v ^ (v>>b); break;
+  case XORSHIFT_SEQ_RRL: v = v ^ (v>>a); v = v ^ (v>>c); v = v ^ (v<<b); break;
+  default: break;
+  }
+  xorshift32_value = v;
+  result = v;
+  return result;
+}
+#endif
