#include "syscall.h"

#ifdef USE_SYSCALL_WRAPPER
void __exit(int status)
{
  __r_exit(0, status);
}
int __read(int fd, void *buffer, int size)
{
  return __r_read(0, fd, buffer, size);
}
int __write(int fd, const void *buffer, int size)
{
  return __r_write(0, fd, buffer, size);
}
int __open(const char *path, int flags, int mode)
{
  return __r_open(0, path, flags, mode);
}
int __close(int fd)
{
  return __r_close(0, fd);
}
#endif

void exit(int status)
{
  __exit(status);
}

int strlen(char *str)
{
  int i;
  for (i = 0; str[i]; i++)
    ;
  return i;
}

int write1(int fd, unsigned char c)
{
  return __write(fd, &c, 1);
}

int putchar(int fd, int c)
{
  write1(fd, c);
  return c;
}

int nputs(int fd, char *str, int len)
{
#ifdef WRITE1
  for (; len > 0; len--)
    write1(fd, *(str++));
#else
  __write(fd, str, len);
#endif
  return 0;
}

int puts(int fd, char *str)
{
  return nputs(fd, str, strlen(str));
}

#ifdef RSHIFT
/*
 * For architecture not to be able to use shift instruction for well,
 * execute shift operation without shift instruction.
 */
unsigned long rshiftn(unsigned long value, int n)
{
  unsigned long mask = 1, bit = 1;
  unsigned long ret = 0;
  int i;

  for (i = 0; i < n; i++)
    mask = mask + mask;

  for (i = 0; i < sizeof(unsigned long) * 8 - n; i++) {
    if (value & mask)
      ret |= bit;
    mask = mask + mask;
    bit = bit + bit; /* Use addition for no using shift and multiplication */
  }

  return ret;
}

unsigned long RSHIFT(unsigned long value)
{
  return rshiftn(value, 4);
}
#endif

int putxval(int fd, unsigned long value, int column)
{
  char buf[17];
  char *p;

  p = buf + sizeof(buf) - 1;
  *(p--) = '\0';

  if (!value && !column)
    column++;

  while (value || column) {
    *(p--) = "0123456789abcdef"[value & 0xf];
    value >>= 4;
    if (column) column--;
  }

  puts(fd, p + 1);

  return 0;
}

int putsxval(int fd, unsigned long value)
{
  int r = putxval(fd, value, 8);
  puts(fd, "\n");
  return r;
}

#include "rnd-bfin-elf.c"
#include "rnd-linear.c"

/* #define ENCRYPT */
unsigned char key[] =
#ifdef ENCRYPT
  "SECCON{YouUseManyDSPWhileNotKnowing}"
#else
  {
    0xf5, 0xa2, 0xd7, 0x7e, 0x7d, 0xcd, 0x7b, 0x60,
    0x11, 0xaa, 0x79, 0x86, 0xef, 0xb6, 0x79, 0x1f,
    0x2f, 0x93, 0x97, 0xfd, 0xb5, 0x1b, 0x59, 0xc5,
    0x4b, 0x81, 0x33, 0x11, 0x71, 0x85, 0x27, 0x96,
    0x6f, 0xa9, 0x93, 0x60, 0x00
  }
#endif
;

int decrypt(unsigned char *dkey, unsigned char *key, int size)
{
  int i;
  for (i = 0; i < size; i++) {
    dkey[i] = key[i] ^ (random_get_value() & 0xff);
  }
  return 0;
}

int main()
{
  unsigned char dkey[80];

  random_set_param0_default();
  random_set_param1_default();
  random_set_seed_default();

  decrypt(dkey, key, sizeof(key) - 1);
  nputs(1, (char *)dkey, sizeof(key) - 1);
  puts(1, "\n");

  exit(0);
  return 0;
}
