#include "syscall.h"

#ifdef USE_SYSCALL_WRAPPER
void __exit(int status)
{
  __r_exit(0, status);
}
int __read(int fd, void *buffer, int size)
{
  return __r_read(0, fd, buffer, size);
}
int __write(int fd, const void *buffer, int size)
{
  return __r_write(0, fd, buffer, size);
}
int __open(const char *path, int flags, int mode)
{
  return __r_open(0, path, flags, mode);
}
int __close(int fd)
{
  return __r_close(0, fd);
}
#endif

void exit(int status)
{
  __exit(status);
}

int strlen(char *str)
{
  int i;
  for (i = 0; str[i]; i++)
    ;
  return i;
}

int write1(int fd, unsigned char c)
{
  return __write(fd, &c, 1);
}

int putchar(int fd, int c)
{
  write1(fd, c);
  return c;
}

int nputs(int fd, char *str, int len)
{
#ifdef WRITE1
  for (; len > 0; len--)
    write1(fd, *(str++));
#else
  __write(fd, str, len);
#endif
  return 0;
}

int puts(int fd, char *str)
{
  return nputs(fd, str, strlen(str));
}

#ifdef RSHIFT
/*
 * For architecture not to be able to use shift instruction for well,
 * execute shift operation without shift instruction.
 */
unsigned long rshiftn(unsigned long value, int n)
{
  unsigned long mask = 1, bit = 1;
  unsigned long ret = 0;
  int i;

  for (i = 0; i < n; i++)
    mask = mask + mask;

  for (i = 0; i < sizeof(unsigned long) * 8 - n; i++) {
    if (value & mask)
      ret |= bit;
    mask = mask + mask;
    bit = bit + bit; /* Use addition for no using shift and multiplication */
  }

  return ret;
}

unsigned long RSHIFT(unsigned long value)
{
  return rshiftn(value, 4);
}
#endif

int putxval(int fd, unsigned long value, int column)
{
  char buf[17];
  char *p;

  p = buf + sizeof(buf) - 1;
  *(p--) = '\0';

  if (!value && !column)
    column++;

  while (value || column) {
    *(p--) = "0123456789abcdef"[value & 0xf];
    value >>= 4;
    if (column) column--;
  }

  puts(fd, p + 1);

  return 0;
}

int putsxval(int fd, unsigned long value)
{
  int r = putxval(fd, value, 8);
  puts(fd, "\n");
  return r;
}

#include "rnd-mn10300-elf.c"
#define RNDALG_XORSHIFT32_DEFAULT_SEQ 1
#include "rnd-xorshift32.c"

/* #define ENCRYPT */
unsigned char key[] =
#ifdef ENCRYPT
  "SECCON{TheTimeOfTheMultimediaCame}"
#else
  {
    0xe9, 0xd0, 0xaf, 0x14, 0xc7, 0xff, 0x3f, 0x1c,
    0x1b, 0x7e, 0xc8, 0x55, 0xd9, 0x31, 0xa3, 0x31,
    0xf5, 0x36, 0x20, 0xaf, 0x4b, 0x43, 0xc2, 0x26,
    0x6a, 0x3f, 0xfa, 0x5e, 0xb9, 0x55, 0xca, 0x18,
    0x83, 0xd9, 0x00
  }
#endif
;

int decrypt(unsigned char *dkey, unsigned char *key, int size)
{
  int i;
  for (i = 0; i < size; i++) {
    dkey[i] = key[i] ^ (xorshift_get_value() & 0xff);
  }
  return 0;
}

int main()
{
  unsigned char dkey[80];

  xorshift_set_param_default();
  xorshift_set_seed_default();

  decrypt(dkey, key, sizeof(key) - 1);
  nputs(1, (char *)dkey, sizeof(key) - 1);
  puts(1, "\n");

  exit(0);
  return 0;
}
