#include "syscall.h"

#ifdef USE_SYSCALL_WRAPPER
void __exit(int status)
{
  __r_exit(0, status);
}
int __read(int fd, void *buffer, int size)
{
  return __r_read(0, fd, buffer, size);
}
int __write(int fd, const void *buffer, int size)
{
  return __r_write(0, fd, buffer, size);
}
int __open(const char *path, int flags, int mode)
{
  return __r_open(0, path, flags, mode);
}
int __close(int fd)
{
  return __r_close(0, fd);
}
#endif

void exit(int status)
{
  __exit(status);
}

int strlen(char *str)
{
  int i;
  for (i = 0; str[i]; i++)
    ;
  return i;
}

int write1(int fd, unsigned char c)
{
  return __write(fd, &c, 1);
}

int putchar(int fd, int c)
{
  write1(fd, c);
  return c;
}

int nputs(int fd, char *str, int len)
{
#ifdef WRITE1
  for (; len > 0; len--)
    write1(fd, *(str++));
#else
  __write(fd, str, len);
#endif
  return 0;
}

int puts(int fd, char *str)
{
  return nputs(fd, str, strlen(str));
}

#ifdef RSHIFT
/*
 * For architecture not to be able to use shift instruction for well,
 * execute shift operation without shift instruction.
 */
unsigned long rshiftn(unsigned long value, int n)
{
  unsigned long mask = 1, bit = 1;
  unsigned long ret = 0;
  int i;

  for (i = 0; i < n; i++)
    mask = mask + mask;

  for (i = 0; i < sizeof(unsigned long) * 8 - n; i++) {
    if (value & mask)
      ret |= bit;
    mask = mask + mask;
    bit = bit + bit; /* Use addition for no using shift and multiplication */
  }

  return ret;
}

unsigned long RSHIFT(unsigned long value)
{
  return rshiftn(value, 4);
}
#endif

int putxval(int fd, unsigned short value, int column)
{
  char buf[17];
  char *p;

  p = buf + sizeof(buf) - 1;
  *(p--) = '\0';

  if (!value && !column)
    column++;

  while (value || column) {
    *(p--) = "0123456789abcdef"[value & 0xf];
    value >>= 4;
    if (column) column--;
  }

  puts(fd, p + 1);

  return 0;
}

int putsxval(int fd, unsigned short value)
{
  int r = putxval(fd, value, 4);
  puts(fd, "\n");
  return r;
}

#include "rnd-msp430-elf.c"
#include "rnd-mseq16.c"

/* #define ENCRYPT */
unsigned char key[] =
#ifdef ENCRYPT
  "SECCON{YouCanSaveElectricityMore}"
#else
  {
    0x74, 0xe9, 0x8f, 0xd7, 0x07, 0x5a, 0x40, 0xf1,
    0x27, 0x9a, 0xa3, 0x34, 0xcd, 0xa7, 0x0b, 0x26,
    0x62, 0xcb, 0xed, 0xba, 0xb1, 0xde, 0x65, 0xe0,
    0x65, 0x40, 0xc5, 0x76, 0xec, 0xe2, 0x4c, 0x38,
    0xc9, 0x00
  }
#endif
;

int decrypt(unsigned char *dkey, unsigned char *key, int size)
{
  int i;
  for (i = 0; i < size; i++) {
    dkey[i] = key[i] ^ (random_get_value(8) & 0xff);
  }
  return 0;
}

int main()
{
  unsigned char dkey[80];

  random_set_param_default();
  random_set_seed_default();

  decrypt(dkey, key, sizeof(key) - 1);
  nputs(1, (char *)dkey, sizeof(key) - 1);
  puts(1, "\n");

  exit(0);
  return 0;
}
