#!/usr/bin/perl

sub getrand {
	my $n;
	($n) = @_;
	return int(rand($n));
}

sub hit {
	my $r;
	($r) = @_;
	return getrand(100) < $r;
}

sub isenable {
	my $c;
	my $n;
	($c, $n) = @_;
	return ($c < 0) || ($c >= $n);
}

sub outconf {
	my $word;
	my $def;
	($word, $def) = @_;
	if (defined($def)) {
		print "#define $word $def\n";
	} else {
		print "#define $word\n";
	}
}

sub outconfxval {
	my $word;
	my $val;
	($word, $val) = @_;
	outconf($word, sprintf("0x%x", $val));
}

$count = shift(@ARGV);
if (!($count =~ /^\d+$/)) {
	$count = -1;
}

################################################################
# Configurations

if (isenable($count,  10)) { if (hit(30)) { outconf("CONFIG_RAND_XOR55"); } }
if (isenable($count,  20)) { if (hit(30)) { outconf("CONFIG_RAND_XORFF"); } }
if (isenable($count,  30)) { if (hit(30)) { outconf("CONFIG_RAND_XORVAL", "0x" . sprintf("%02x", getrand(255) + 1)); } }
if (isenable($count,  40)) { if (hit(30)) { outconf("CONFIG_RAND_XORAA"); } }
if (isenable($count,  50)) { if (hit(30)) { outconf("CONFIG_DUMMY_FUNC"); } }

if (isenable($count,  60)) { if (hit(30)) {
	$n = getrand(16) + 1;
	$val = "";
	for ($i = 0; $i < $n; $i++) {
		$val .= sprintf("0x%08x, ", getrand(0xffffffff) + 1);
	}
	outconf("CONFIG_DUMMY_AFTER", "{ $val}");
}}
if (isenable($count,  80)) { if (hit(30)) {
	$n = getrand(16) + 1;
	$val = "";
	for ($i = 0; $i < $n; $i++) {
		$val .= sprintf("0x%08x, ", getrand(0xffffffff) + 1);
	}
	outconf("CONFIG_DUMMY_BEFORE", "{ $val}");
}}

if (isenable($count, 100)) { if (hit(30)) { outconf("CONFIG_RAND_REVEND"); } }
if (isenable($count, 120)) { if (hit(30)) { outconf("CONFIG_RAND_ROTATE"); } }
if (isenable($count, 140)) { if (hit(30)) { outconf("CONFIG_RAND_MERGEBITS"); } }
if (isenable($count, 160)) { if (hit(30)) { outconf("CONFIG_RAND_REV"); } }
if (isenable($count, 180)) { if (hit(30)) { outconf("CONFIG_RAND_SKIP", getrand(16) + 1); } }
if (isenable($count, 200)) { if (hit(30)) { outconf("CONFIG_RAND_VOLATILE"); } }
if (isenable($count, 230)) { if (hit(30)) { outconf("CONFIG_GETRAND_VOLATILE"); } }
if (isenable($count, 260)) { if (hit(30)) { outconf("CONFIG_RAND_SAVEVAL"); } }
if (isenable($count, 290)) { if (hit(30)) { outconf("CONFIG_RAND_DUP", getrand(16) + 1); } }
if (isenable($count, 320)) { if (hit(30)) { outconf("CONFIG_DECRYPT_VOLATILE"); } }
if (isenable($count, 360)) { if (hit(30)) { outconf("CONFIG_RAND_DUP2", getrand(16) + 1); } }
if (isenable($count, 400)) { if (hit(30)) { outconf("CONFIG_DECRYPT_LOOP_VOLATILE"); } }

################################################################
# Parameters of linear

$rate = 100;
if (isenable($count, 40)) { $rate = 30; }

if (hit($rate)) { outconfxval("CONFIG_LINEAR_PARAM0", getrand(100000) + 10000); }
if (hit($rate)) { outconfxval("CONFIG_LINEAR_PARAM1", getrand(100000) + 10000); }
if (hit($rate)) { outconfxval("CONFIG_LINEAR_SEED",   getrand(100000) + 10000); }

################################################################
# Parameters of M-sequence

@mseq_param = (
	0x0860, 0x0c20, 0x0052, 0x9400, 0x2050, 0x1408, 0x0b00, 0x01a0,
	0x0290, 0x1280, 0x2104, 0x4108, 0x1082, 0x8210, 0x2240, 0x0488);

$rate = 100;
if (isenable($count, 40)) { $rate = 30; }

if (hit($rate)) {
	$n = @mseq_param;
	$h = sprintf("(0x%04x)", $mseq_param[getrand($n)]);
	outconf("CONFIG_MSEQ_PARAM", $h);
}
if (hit($rate)) { outconfxval("CONFIG_MSEQ_SEED", getrand(0xffff) + 1); }
if (hit($rate)) { outconf("CONFIG_MSEQ_SHIFT8"); }
if (hit($rate)) { outconf("CONFIG_MSEQ_SHIFT", getrand(16) + 1); }

################################################################
# Parameters of xorshift

@xorshift32_param = (
  "1,3,10", "1,5,16", "1,5,19", "1,9,29", "1,11,6", "1,11,16", "1,19,3", "1,21,20", "1,27,27",
  "2,5,15", "2,5,21", "2,7,7", "2,7,9", "2,7,25", "2,9,15", "2,15,17", "2,15,25", "2,21,9",
  "3,1,14", "3,3,26", "3,3,28", "3,3,29", "3,5,20", "3,5,22", "3,5,25", "3,7,29", "3,13,7",
  "3,23,25", "3,25,24", "3,27,11", "4,3,17", "4,3,27", "4,5,15", "5,3,21", "5,7,22", "5,9,7",
  "5,9,28", "5,9,31", "5,13,6", "5,15,17", "5,17,13", "5,21,12", "5,27,8", "5,27,21", "5,27,25",
  "5,27,28", "6,1,11", "6,3,17", "6,17,9", "6,21,7", "6,21,13", "7,1,9", "7,1,18", "7,1,25",
  "7,13,25", "7,17,21", "7,25,12", "7,25,20", "8,7,23", "8,9,23", "9,5,25", "9,11,19", #"9,5,1"
  "9,21,16", "10,9,21", "10,9,25", "11,7,12", "11,7,16", "11,17,13", "11,21,13", "12,9,23", "13,3,17",
  "13,3,27", "13,5,19", "13,17,15", "14,1,15", "14,13,15", "15,1,29", "17,15,20", "17,15,23", "17,15,26", 
);

@xorshift64_param = (
  "1,1,54", "1,13,45", "1,35,11", "2,43,27", "3,29,40", "3,53,7",
  "4,31,33", "4,53,7", "5,47,23", "6,27,7", "7,11,10", "7,41,40",
  "8,25,59", "9,19,18", "9,41,45", "11,9,34", "11,53,23", "13,3,40",
  "13,21,18", "14,23,33", "15,21,46", "16,13,55", "17,27,22", "18,41,23",
  "20,1,31", "21,17,48", "22,3,39", "23,17,56", "25,7,49", "25,27,53",
  "33,31,43",
);

$rate = 100;
if (isenable($count, 70)) { $rate = 30; }

if (hit($rate)) {
	$n = @xorshift32_param;
	($a, $b, $c) = split(',', $xorshift32_param[getrand($n)]);
	$seq = 0;
	if (hit(30)) { $seq = getrand(4); }
	if (hit(30)) { $t = $c; $c = $a; $a = $t; }
	outconf("CONFIG_XORSHIFT32_PARAM_SEQ", $seq);
	outconf("CONFIG_XORSHIFT32_PARAM_A", $a);
	outconf("CONFIG_XORSHIFT32_PARAM_B", $b);
	outconf("CONFIG_XORSHIFT32_PARAM_C", $c);
}
if (hit($rate)) { outconfxval("CONFIG_XORSHIFT32_SEED", getrand(0xffffffff) + 1); }

$rate = 100;
if (isenable($count, 120)) { $rate = 30; }

if (hit($rate)) {
	$n = @xorshift64_param;
	($a, $b, $c) = split(',', $xorshift64_param[getrand($n)]);
	$seq = 0;
	if (hit(30)) { $seq = getrand(4); }
	if (hit(30)) { $t = $c; $c = $a; $a = $t; }
	outconf("CONFIG_XORSHIFT64_PARAM_SEQ", $seq);
	outconf("CONFIG_XORSHIFT64_PARAM_A", $a);
	outconf("CONFIG_XORSHIFT64_PARAM_B", $b);
	outconf("CONFIG_XORSHIFT64_PARAM_C", $c);
}
if (hit($rate)) { outconfxval("CONFIG_XORSHIFT64_SEED", getrand(0xffffffff) + 1); }

################################################################
# End of file
