#include "syscall.h"
#include "config.h"
#include "archdep.h"

#ifdef USE_SYSCALL_WRAPPER
void __exit(int status)
{
  __r_exit(0, status);
}
int __read(int fd, void *buffer, int size)
{
  return __r_read(0, fd, buffer, size);
}
int __write(int fd, const void *buffer, int size)
{
  return __r_write(0, fd, buffer, size);
}
int __open(const char *path, int flags, int mode)
{
  return __r_open(0, path, flags, mode);
}
int __close(int fd)
{
  return __r_close(0, fd);
}
#endif

void exit(int status)
{
  __exit(status);
}

int strlen(char *str)
{
  int i;
  for (i = 0; str[i]; i++)
    ;
  return i;
}

int write1(int fd, unsigned char c)
{
  return __write(fd, &c, 1);
}

int putchar(int fd, int c)
{
  write1(fd, c);
  return c;
}

int nputs(int fd, char *str, int len)
{
#ifdef WRITE1
  for (; len > 0; len--)
    write1(fd, *(str++));
#else
  __write(fd, str, len);
#endif
  return 0;
}

int puts(int fd, char *str)
{
  return nputs(fd, str, strlen(str));
}

#ifdef RSHIFT
/*
 * For architecture not to be able to use shift instruction for well,
 * execute shift operation without shift instruction.
 */
unsigned long rshiftn(unsigned long value, int n)
{
  unsigned long mask = 1, bit = 1;
  unsigned long ret = 0;
  int i;

  for (i = 0; i < n; i++)
    mask = mask + mask;

  for (i = 0; i < sizeof(unsigned long) * 8 - n; i++) {
    if (value & mask)
      ret |= bit;
    mask = mask + mask;
    bit = bit + bit; /* Use addition for no using shift and multiplication */
  }

  return ret;
}

unsigned long RSHIFT(unsigned long value)
{
  return rshiftn(value, 4);
}
#endif

int putxval(int fd, unsigned long value, int column)
{
  char buf[17];
  char *p;

  p = buf + sizeof(buf) - 1;
  *(p--) = '\0';

  if (!value && !column)
    column++;

  while (value || column) {
    *(p--) = "0123456789abcdef"[value & 0xf];
    value >>= 4;
    if (column) column--;
  }

  puts(fd, p + 1);

  return 0;
}

#ifdef __bfin__
#include "rnd-bfin-elf.c"
#endif
#ifdef __MSP430__
#include "rnd-msp430-elf.c"
#endif
#ifdef __mn10300__
#include "rnd-mn10300-elf.c"
#endif
#ifdef __v850__
#include "rnd-v850-elf.c"
#endif
#ifdef __mips64
#include "rnd-mips64-elf.c"
#endif
#ifdef __arm__
#include "rnd-arm-elf.c"
#endif
#ifdef __RL78__
#include "rnd-rl78-elf.c"
#endif
#ifdef __microblaze__
#include "rnd-microblaze-elf.c"
#endif
#if defined(__mips) && !defined(__mips64)
#include "rnd-mips-elf.c"
#endif
#ifdef __aarch64__
#include "rnd-aarch64-elf.c"
#endif

#ifdef RNDALG_LINEAR
#include "rnd-linear.c"
#endif
#ifdef RNDALG_MSEQ16
#include "rnd-mseq16.c"
#endif
#ifdef RNDALG_XORSHIFT32
#include "rnd-xorshift32.c"
#endif
#ifdef RNDALG_XORSHIFT64
#include "rnd-xorshift64.c"
#endif

#ifdef CONFIG_DUMMY_FUNC
int dummy(int *val)
{
  return *val = 1;
}
#endif

unsigned int rand()
{
#ifdef CONFIG_RAND_VOLATILE
  volatile
#endif
  unsigned int r;
#ifdef CONFIG_DUMMY_FUNC
  int val;
  dummy(&val);
#endif
#ifdef RNDALG_LINEAR
  r = random_get_value();
#endif
#ifdef RNDALG_MSEQ
#ifdef CONFIG_MSEQ_SHIFT8
  r = random_get_value(8);
#elif defined(CONFIG_MSEQ_SHIFT)
  r = random_get_value(CONFIG_MSEQ_SHIFT);
#else
  r = random_get_value(16);
#endif
#endif
#ifdef RNDALG_XORSHIFT
  r = xorshift_get_value();
#endif
#ifdef CONFIG_RAND_XORAA
  r ^= 0xaaaaaaaa;
#endif
  return r;
}

#ifdef RSHIFT
#ifdef CONFIG_RAND_SAVEVAL
#undef CONFIG_RAND_SAVEVAL /* avoid many loop for shift */
#endif
#endif

unsigned char getrand()
{
#ifdef CONFIG_GETRAND_VOLATILE
  volatile
#endif
  int r;

#ifdef CONFIG_RAND_SAVEVAL
  static unsigned int val = 0;
  if (!val) val = rand();
  r = val & 0xff;
  val >>= 8;
#else
  r = rand() & 0xff;
#endif

  return r;
}

#ifdef CONFIG_RAND_SKIP
unsigned char getnrand()
{
  int i;
  unsigned char c;
  for (i = 0; i < CONFIG_RAND_SKIP; i++)
    c = getrand();
  return c;
}
#define getrand getnrand
#endif

#ifdef CONFIG_DUMMY_BEFORE
unsigned long dummy_before[] = CONFIG_DUMMY_BEFORE;
#endif

unsigned char key[] =
#include "key.h"
;

#ifdef CONFIG_DUMMY_AFTER
unsigned long dummy_after[] = CONFIG_DUMMY_AFTER;
#endif

int decrypt(unsigned char *dkey, unsigned char *key, int size)
{
#ifdef CONFIG_DECRYPT_LOOP_VOLATILE
  volatile
#endif
  int i;
#ifdef CONFIG_DECRYPT_VOLATILE
  volatile
#endif
  unsigned char r;
#ifdef CONFIG_RAND_DUP
#ifdef CONFIG_DECRYPT_LOOP_VOLATILE
  volatile
#endif
  int j;
#endif
#ifdef CONFIG_RAND_DUP2
#ifdef CONFIG_DECRYPT_LOOP_VOLATILE
  volatile
#endif
  int k;
  for (k = 0; k < CONFIG_RAND_DUP2; k++)
#endif
#ifdef CONFIG_RAND_REV
  for (i = size - 1; i >= 0; i--) {
#else
  for (i = 0; i < size; i++) {
#endif
#ifdef CONFIG_RAND_DUP
    for (j = 0; j < CONFIG_RAND_DUP; j++)
#endif
    {
      r = getrand();
#ifdef CONFIG_RAND_REVEND
      r = ((r << 4) & 0xf0) | ((r >> 4) & 0x0f);
#endif
#ifdef CONFIG_RAND_ROTATE
      r = ((r & 1) ? 0x80 : 0) | (r >> 1);
#endif
#ifdef CONFIG_RAND_MERGEBITS
      r = ((r << 1) & 0xaa) | ((r >> 1) & 0x55);
#endif
#ifdef CONFIG_RAND_XOR55
      r ^= 0x55;
#endif
#ifdef CONFIG_RAND_XORFF
      r ^= 0xff;
#endif
#ifdef CONFIG_RAND_XORVAL
      r ^= CONFIG_RAND_XORVAL;
#endif
      dkey[i] = key[i] ^ r;
    }
  }
  return 0;
}

int main()
{
  unsigned char dkey[sizeof(key)];

#ifdef RNDALG_LINEAR
#ifdef CONFIG_LINEAR_PARAM0
  random_param0(CONFIG_LINEAR_PARAM0);
#else
  random_set_param0_default();
#endif
#ifdef CONFIG_LINEAR_PARAM1
  random_param1(CONFIG_LINEAR_PARAM1);
#else
  random_set_param1_default();
#endif
#ifdef CONFIG_LINEAR_SEED
  random_seed(CONFIG_LINEAR_SEED);
#else
  random_set_seed_default();
#endif
#endif

#ifdef RNDALG_MSEQ
#ifdef CONFIG_MSEQ_PARAM
  random_param(CONFIG_MSEQ_PARAM);
#else
  random_set_param_default();
#endif
#ifdef CONFIG_MSEQ_SEED
  random_seed(CONFIG_MSEQ_SEED);
#else
  random_set_seed_default();
#endif
#endif

#ifdef RNDALG_XORSHIFT32
#ifdef CONFIG_XORSHIFT32_PARAM
  xorshift_set_param(CONFIG_XORSHIFT32_PARAM_SEQ,
		     CONFIG_XORSHIFT32_PARAM_A,
		     CONFIG_XORSHIFT32_PARAM_B,
		     CONFIG_XORSHIFT32_PARAM_C);
#else
  xorshift_set_param_default();
#endif
#ifdef CONFIG_XORSHIFT32_SEED
  xorshift_seed(CONFIG_XORSHIFT32_SEED);
#else
  xorshift_set_seed_default();
#endif
#endif

#ifdef RNDALG_XORSHIFT64
#ifdef CONFIG_XORSHIFT64_PARAM
  xorshift_set_param(CONFIG_XORSHIFT64_PARAM_SEQ,
		     CONFIG_XORSHIFT64_PARAM_A,
		     CONFIG_XORSHIFT64_PARAM_B,
		     CONFIG_XORSHIFT64_PARAM_C);
#else
  xorshift_set_param_default();
#endif
#ifdef CONFIG_XORSHIFT64_SEED
  xorshift_seed(CONFIG_XORSHIFT64_SEED);
#else
  xorshift_set_seed_default();
#endif
#endif

  decrypt(dkey, key, sizeof(key) - 1);
  nputs(1, (char *)dkey, sizeof(key) - 1);
  puts(1, "\n");

  exit(0);
  return 0;
}
