#!/bin/sh

#keyboard=jp106
#keylayout=jp

#password=yes

#ntp=192.168.1.1

#interface=eth0
#address=192.168.1.2
#netmask=255.255.255.0
#gateway=192.168.1.1

LANG=C

date

. /home/user/server/files/param.sh

################################################################
# Setup keyboard

if [ "$keyboard" ]; then
localectl set-keymap $keyboard
echo "KEYTABLE=$keyboard" >> /etc/default/keyboard
echo "MODEL=$keyboard" >> /etc/default/keyboard
echo "LAYOUT=$keylayout" >> /etc/default/keyboard
fi

################################################################
# Setting passwords

if [ "$password" ]; then
passwd root
passwd user
fi

################################################################
# Make users
# Not set password. If password is not set, the user cannot login.

cd $homedir

groupadd -g 10000 arch

for u in $allusers; do
    archparam $u
    useradd -u $uid -d /home/$user -g arch -c $user -m $user
#    chown root:wheel /home/$user
#    chmod 755 /home/$user
    mkdir /home/$user/bin
    mkdir /home/$user/lib
#    mkdir /home/$user/dev
    mkdir /home/$user/tmp
    chmod 777 /home/$user/tmp
    touch /home/$user/tmp/.nodelete
    chattr +i /home/$user/tmp/.nodelete
done

################################################################
# Setup directory

cd $homedir

mkdir /home/lib
mkdir /home/bin
mkdir /home/bin/gdb
mkdir /home/bin/exec

cp /bin/sh /home/bin
cp /bin/sleep /home/bin
cp $serverdir/files/rungdb.sh /home/bin

cp /lib/ld-linux.so.* /home/lib
#cp /lib/linux-gate.so.* /home/lib

ldd $crossdir/bin/*-gdb $crossdir/bin/*-run /bin/sh /bin/sleep \
        | grep "=>" | awk '{print $3}' | sort | uniq \
        | xargs -I % cp % /home/lib

#mkdir /home/dev
#mknod -m 644 /home/dev/urandom c 1 9

################################################################
# Setup GDB

cd $homedir

cp -R $serverdir/exec-gdb $homedir
cp -R $serverdir/exec-sim $homedir

chown -R user:user $execgdbdir
chown -R user:user $execsimdir

cp $execgdbdir/*.x /home/bin/exec

cp $crossdir/bin/*-gdb /home/bin/gdb
cp $crossdir/bin/*-run /home/bin/gdb

################################################################
# Fix files

cd $homedir

chattr -R +i /home/bin /home/lib
#chattr -R +i /home/dev

################################################################
# Setup iptables

cd $homedir

export DEBIAN_FRONTEND=noninteractive
export APT_LISTCHANGES_FRONTEND=none

yes | apt-get -y install iptables
yes | apt-get -y install iptables-persistent

mv /etc/iptables/rules.v4 /etc/iptables/rules.v4.orig
# copy for saving file attributes.
cp /etc/iptables/rules.v4.orig /etc/iptables/rules.v4
echo "*filter" > /etc/iptables/rules.v4

echo "-A INPUT -m state --state ESTABLISHED,RELATED -j ACCEPT" >> /etc/iptables/rules.v4
echo "-A INPUT -p icmp -j ACCEPT" >> /etc/iptables/rules.v4
echo "-A INPUT -i lo -j ACCEPT" >> /etc/iptables/rules.v4

echo "-A INPUT -m state --state NEW -m tcp -p tcp --dport 22 -j ACCEPT" \
    >> /etc/iptables/rules.v4
#echo "-A INPUT -m state --state NEW -m tcp -p tcp --dport 80 -j ACCEPT" \
#    >> /etc/iptables/rules.v4
echo "-A INPUT -m state --state NEW -m hashlimit -p tcp --dport 80 --hashlimit-name http --hashlimit 1/s --hashlimit-burst 10 --hashlimit-mode srcip --hashlimit-srcmask 32 --hashlimit-htable-expire 10000 -j ACCEPT" \
    >> /etc/iptables/rules.v4

#echo "-A INPUT -m state --state NEW -m tcp -p tcp --dport 9999 -j ACCEPT" \
#    >> /etc/iptables/rules.v4
echo "-A INPUT -m state --state NEW -m hashlimit -p tcp --dport 9999 --hashlimit-name commit --hashlimit 1/s --hashlimit-burst 10 --hashlimit-mode srcip --hashlimit-srcmask 32 --hashlimit-htable-expire 10000 -j ACCEPT" \
    >> /etc/iptables/rules.v4

for u in $allusers; do
    archparam $u
#    echo "-A INPUT -m state --state NEW -m tcp -p tcp --dport $port -j ACCEPT" \
#        >> /etc/iptables/rules.v4
    echo "-A INPUT -m state --state NEW -m hashlimit -p tcp --dport $port --hashlimit-name port$port --hashlimit 1/s --hashlimit-burst 3 --hashlimit-mode srcip --hashlimit-srcmask 32 --hashlimit-htable-expire 10000 -j ACCEPT" \
        >> /etc/iptables/rules.v4
done

echo "-A INPUT -j REJECT --reject-with icmp-host-prohibited" \
    >> /etc/iptables/rules.v4
echo "-A FORWARD -j REJECT --reject-with icmp-host-prohibited" \
    >> /etc/iptables/rules.v4

echo "-A INPUT -j REJECT" >> /etc/iptables/rules.v4
echo "-A FORWARD -j REJECT" >> /etc/iptables/rules.v4
echo "-A OUTPUT -j ACCEPT" >> /etc/iptables/rules.v4

echo COMMIT >> /etc/iptables/rules.v4

################################################################
# Setup SSH

cd $homedir

yes | apt-get -y install openssh-server

mv /etc/ssh/sshd_config /etc/ssh/sshd_config.orig
# copy for saving file attributes.
cp /etc/ssh/sshd_config.orig /etc/ssh/sshd_config
cat /etc/ssh/sshd_config.orig \
    | $tocomment - - 0 0 "#" ^PermitRootLogin \
    | $tocomment - - 0 0 "#" ^PasswordAuthentication \
    | $tocomment - - 0 0 "#" ^PermitEmptyPasswords \
    | $tocomment - - 0 0 "#" ^ChallengeResponseAuthentication \
    > /etc/ssh/sshd_config
echo "" >> /etc/ssh/sshd_config
echo "PermitRootLogin no" >> /etc/ssh/sshd_config
echo "PasswordAuthentication no" >> /etc/ssh/sshd_config
echo "PermitEmptyPasswords no" >> /etc/ssh/sshd_config
echo "ChallengeResponseAuthentication yes" >> /etc/ssh/sshd_config
echo "AllowUsers user" >> /etc/ssh/sshd_config

################################################################
# Setup httpd

cd $homedir

yes | apt-get -y install apache2

cp $serverdir/files/index.html /var/www/html
echo "Team flags:" > /var/www/html/flag.txt
chmod 666 /var/www/html/flag.txt

touch /var/www/html/runme.x
chmod 666 /var/www/html/runme.x
#chattr +a /var/www/html/flag.txt

for u in $users; do
    archparam $u
    cp $serverdir/exec-gdb/$arch.x /var/www/html
    chmod -x /var/www/html/$arch.x
done

cp $archivedir/binutils-2.28.tar.bz2 /var/www/html
cp $archivedir/gcc-4.9.4.tar.bz2 /var/www/html
cp $archivedir/gdb-7.12.1.tar.gz /var/www/html
cp $archivedir/gmp-6.1.2.tar.bz2 /var/www/html
cp $archivedir/mpfr-3.1.5.tar.bz2 /var/www/html
cp $archivedir/mpc-1.0.3.tar.gz /var/www/html
cp $archivedir/cross-gcc494-v1.0.zip /var/www/html

################################################################
# Setup ntp

if [ "$ntp" ]; then

cd $homedir

yes | apt-get -y install ntp

mv /etc/ntp.conf /etc/ntp.conf.orig
# copy for saving file attributes.
cp /etc/ntp.conf.orig /etc/ntp.conf
cat /etc/ntp.conf.orig \
    | $tocomment - - 0 0 "#" ^server \
    > /etc/ntp.conf
echo "" >> /etc/ntp.conf
echo "server $ntp" >> /etc/ntp.conf

fi

################################################################
# Setup network interface

if [ "$interface" ]; then

cd $homedir

mv /etc/network/interfaces /etc/network/interfaces.orig
# copy for saving file attributes.
cp /etc/network/interfaces.orig /etc/network/interfaces
cat /etc/network/interfaces.orig \
    | $tocomment - - 0 0 "#" dhcp \
    > /etc/network/interfaces
echo "" >> /etc/network/interfaces
echo "auto $interface" >> /etc/network/interfaces
echo "iface $interface inet static" >> /etc/network/interfaces
echo "address $address" >> /etc/network/interfaces
echo "netmask $netmask" >> /etc/network/interfaces
echo "gateway $gateway" >> /etc/network/interfaces

fi

################################################################
# Setup working files

cd $homedir
mkdir $homedir/tmp
#chmod 777 $homedir/tmp

# Limit of simple-inetd (relay to commit)
echo "256" > $homedir/tmp/size.txt

# Limit of simple-inetd-connect (relay to GDB)
echo "5000" > $homedir/tmp/size-connect.txt

# Counter of commit
echo "0" > $homedir/tmp/count.txt

# Update interval
echo "5" > $homedir/tmp/interval.txt

touch $homedir/tmp/error.log

chown -R user:user $homedir/tmp

cp $serverdir/files/commit.sh $homedir
chmod +x $homedir/commit.sh
chown user:user $homedir/commit.sh

################################################################
# Setup boot

cd $homedir

mv /etc/rc.local /etc/rc.local.orig
# copy for saving file attributes. (need execution bit to run rc.local)
cp /etc/rc.local.orig /etc/rc.local
cat /etc/rc.local.orig \
    | $tocomment - - 0 0 "#" ^exit \
    > /etc/rc.local
echo "/home/user/server/files/mount.sh" >> /etc/rc.local
echo "/home/user/server/files/run.sh &" >> /etc/rc.local
echo "exit 0" >> /etc/rc.local

################################################################
# Shutdown

date

/sbin/shutdown -h now
