#include "syscall.h"

#ifdef USE_SYSCALL_WRAPPER
void __exit(int status)
{
  __r_exit(0, status);
}
int __read(int fd, void *buffer, int size)
{
  return __r_read(0, fd, buffer, size);
}
int __write(int fd, const void *buffer, int size)
{
  return __r_write(0, fd, buffer, size);
}
int __open(const char *path, int flags, int mode)
{
  return __r_open(0, path, flags, mode);
}
int __close(int fd)
{
  return __r_close(0, fd);
}
#endif

void exit(int status)
{
  __exit(status);
}

int strlen(char *str)
{
  int i;
  for (i = 0; str[i]; i++)
    ;
  return i;
}

int write1(int fd, unsigned char c)
{
  return __write(fd, &c, 1);
}

int putchar(int fd, int c)
{
  write1(fd, c);
  return c;
}

int nputs(int fd, char *str, int len)
{
#ifdef WRITE1
  for (; len > 0; len--)
    write1(fd, *(str++));
#else
  __write(fd, str, len);
#endif
  return 0;
}

int puts(int fd, char *str)
{
  return nputs(fd, str, strlen(str));
}

#ifdef RSHIFT
/*
 * For architecture not to be able to use shift instruction for well,
 * execute shift operation without shift instruction.
 */
unsigned long rshift1(unsigned long value)
{
  unsigned long mask, bit = 1;
  unsigned long ret = 0;
  int i;

  for (i = 0; i < sizeof(unsigned long) * 8 - 1; i++) {
    mask = bit + bit; /* Use addition for no using shift and multiplication */
    if (value & mask)
      ret |= bit;
    bit = bit + bit;
  }

  return ret;
}

unsigned long RSHIFT(unsigned long value)
{
  int num = 4;
  for (; num > 0; num--)
    value = rshift1(value);
  return value;
}
#endif

int putxval(int fd, unsigned long value, int column)
{
  char buf[17];
  char *p;

  p = buf + sizeof(buf) - 1;
  *(p--) = '\0';

  if (!value && !column)
    column++;

  while (value || column) {
    *(p--) = "0123456789abcdef"[value & 0xf];
    value >>= 4;
    if (column) column--;
  }

  puts(fd, p + 1);

  return 0;
}

int putsxval(int fd, unsigned long value)
{
  int r = putxval(fd, value, 8);
  puts(fd, "\n");
  return r;
}

#include "rnd-arm-elf.c"
#include "rnd-linear.c"

int main()
{
  int fd = 1, i;
  unsigned int value;

  puts(fd, "Hello World!\n");

  puts(fd, "param0 set test (00000000 00000000 00000001)\n");
  value = random_param0(0);   putsxval(fd, value);
  value = random_param0(1);   putsxval(fd, value);
  value = random_param0(2);   putsxval(fd, value);

  puts(fd, "param1 set test (00000000 00000000 00000001)\n");
  value = random_param1(0);   putsxval(fd, value);
  value = random_param1(1);   putsxval(fd, value);
  value = random_param1(2);   putsxval(fd, value);

  puts(fd, "seed set test (00000000 00000000 00000001)\n");
  value = random_seed(0);     putsxval(fd, value);
  value = random_seed(1);     putsxval(fd, value);
  value = random_seed(2);     putsxval(fd, value);

  puts(fd, "param set (00000002 00000002 00000002)\n");
  value = random_param0(2);   putsxval(fd, value);
  value = random_param1(1);   putsxval(fd, value);
  value = random_seed(1);     putsxval(fd, value);
  puts(fd, "value test (00000003 00000007 0000000f)\n");
  value = random_get_value(); putsxval(fd, value);
  value = random_get_value(); putsxval(fd, value);
  value = random_get_value(); putsxval(fd, value);

  puts(fd, "param set (00000002 00000001 0000000f)\n");
  value = random_param0(1);   putsxval(fd, value);
  value = random_param1(2);   putsxval(fd, value);
  value = random_seed(1);     putsxval(fd, value);
  puts(fd, "value test (00000003 00000005 00000007)\n");
  value = random_get_value(); putsxval(fd, value);
  value = random_get_value(); putsxval(fd, value);
  value = random_get_value(); putsxval(fd, value);

  puts(fd, "set default (00000001 00000002 00000007)\n");
  value = random_set_param0_default(); putsxval(fd, value);
  value = random_set_param1_default(); putsxval(fd, value);
  value = random_set_seed_default();   putsxval(fd, value);
  puts(fd, "default test ()\n");
  for (i = 0; i < 20; i++) {
    value = random_get_value(); putsxval(fd, value);
  }

  exit(0);
  return 0;
}
