#include "syscall.h"

#ifdef USE_SYSCALL_WRAPPER
void __exit(int status)
{
  __r_exit(0, status);
}
int __read(int fd, void *buffer, int size)
{
  return __r_read(0, fd, buffer, size);
}
int __write(int fd, const void *buffer, int size)
{
  return __r_write(0, fd, buffer, size);
}
int __open(const char *path, int flags, int mode)
{
  return __r_open(0, path, flags, mode);
}
int __close(int fd)
{
  return __r_close(0, fd);
}
#endif

void exit(int status)
{
  __exit(status);
}

int strlen(char *str)
{
  int i;
  for (i = 0; str[i]; i++)
    ;
  return i;
}

int write1(int fd, unsigned char c)
{
  return __write(fd, &c, 1);
}

int putchar(int fd, int c)
{
  write1(fd, c);
  return c;
}

int nputs(int fd, char *str, int len)
{
#ifdef WRITE1
  for (; len > 0; len--)
    write1(fd, *(str++));
#else
  __write(fd, str, len);
#endif
  return 0;
}

int puts(int fd, char *str)
{
  return nputs(fd, str, strlen(str));
}

#ifdef RSHIFT
/*
 * For architecture not to be able to use shift instruction for well,
 * execute shift operation without shift instruction.
 */
unsigned long rshift1(unsigned long value)
{
  unsigned long mask, bit = 1;
  unsigned long ret = 0;
  int i;

  for (i = 0; i < sizeof(unsigned long) * 8 - 1; i++) {
    mask = bit + bit; /* Use addition for no using shift and multiplication */
    if (value & mask)
      ret |= bit;
    bit = bit + bit;
  }

  return ret;
}

unsigned long RSHIFT(unsigned long value)
{
  int num = 4;
  for (; num > 0; num--)
    value = rshift1(value);
  return value;
}
#endif

int putxval(int fd, unsigned long value, int column)
{
  char buf[17];
  char *p;

  p = buf + sizeof(buf) - 1;
  *(p--) = '\0';

  if (!value && !column)
    column++;

  while (value || column) {
    *(p--) = "0123456789abcdef"[value & 0xf];
    value >>= 4;
    if (column) column--;
  }

  puts(fd, p + 1);

  return 0;
}

int putsxval(int fd, unsigned long value)
{
  int r = putxval(fd, value, 16);
  puts(fd, "\n");
  return r;
}

#include "rnd-mips64-elf.c"
#define RNDALG_XORSHIFT64_DEFAULT_SEQ 2
#include "rnd-xorshift64.c"

int main()
{
  int fd = 1, i;
  unsigned long value;

  puts(fd, "Hello World!\n");

  puts(fd, "param set test (...00 ...00 ...01)\n");
  value = xorshift_param(0);    putsxval(fd, value);
  value = xorshift_param(1);    putsxval(fd, value);
  value = xorshift_param(2);    putsxval(fd, value);

  puts(fd, "seed set test (...00 ...00 ...01)\n");
  value = xorshift_seed(0);     putsxval(fd, value);
  value = xorshift_seed(1);     putsxval(fd, value);
  value = xorshift_seed(2);     putsxval(fd, value);

  puts(fd, "param set (LRL) (...02 ...02)\n");
  value = xorshift_set_param(0, 1, 1, 1); putsxval(fd, value);
  value = xorshift_seed(1);     putsxval(fd, value);
  puts(fd, "value test (...06 ...11 ...7e)\n");
  value = xorshift_get_value(); putsxval(fd, value);
  value = xorshift_get_value(); putsxval(fd, value);
  value = xorshift_get_value(); putsxval(fd, value);

  puts(fd, "param set (RLR) (...4104 ...7e)\n");
  value = xorshift_set_param(1, 1, 1, 1); putsxval(fd, value);
  value = xorshift_seed(0x8000000000000000ULL); putsxval(fd, value);
  puts(fd, "value test (60... 88... 7e...)\n");
  value = xorshift_get_value(); putsxval(fd, value);
  value = xorshift_get_value(); putsxval(fd, value);
  value = xorshift_get_value(); putsxval(fd, value);

  puts(fd, "param set (LLR) (...4105 7e...)\n");
  value = xorshift_set_param(2, 1, 1, 1); putsxval(fd, value);
  value = xorshift_seed(1);     putsxval(fd, value);
  puts(fd, "value test (...07 ...16 ...69)\n");
  value = xorshift_get_value(); putsxval(fd, value);
  value = xorshift_get_value(); putsxval(fd, value);
  value = xorshift_get_value(); putsxval(fd, value);

  puts(fd, "param set (RRL) (...4106 ...69)\n");
  value = xorshift_set_param(3, 1, 1, 1); putsxval(fd, value);
  value = xorshift_seed(0x8000000000000000UL); putsxval(fd, value);
  puts(fd, "value test (e0... 68... 96...)\n");
  value = xorshift_get_value(); putsxval(fd, value);
  value = xorshift_get_value(); putsxval(fd, value);
  value = xorshift_get_value(); putsxval(fd, value);

  puts(fd, "param set (a=32, b=16, c=1) (...4107 96...)\n");
  value = xorshift_set_param(0, 32, 16, 1); putsxval(fd, value);
  value = xorshift_seed(1);     putsxval(fd, value);
  puts(fd, "value test (0000000300030003)\n");
  value = xorshift_get_value(); putsxval(fd, value);

  puts(fd, "seed set (7) (...5080 ...300030003)\n");
  value = xorshift_set_param(0, 32, 16, 4); putsxval(fd, value);
  value = xorshift_seed(7);     putsxval(fd, value);
  puts(fd, "value test (0000007700770077)\n");
  value = xorshift_get_value(); putsxval(fd, value);

  puts(fd, "set default (...11080 0000007700770077)\n");
  value = xorshift_set_param_default(); putsxval(fd, value);
  value = xorshift_set_seed_default();  putsxval(fd, value);
  puts(fd, "default test ()\n");
  for (i = 0; i < 20; i++) {
    value = xorshift_get_value(); putsxval(fd, value);
  }

  exit(0);
  return 0;
}
