#include "syscall.h"

#ifdef USE_SYSCALL_WRAPPER
void __exit(int status)
{
  __r_exit(0, status);
}
int __read(int fd, void *buffer, int size)
{
  return __r_read(0, fd, buffer, size);
}
int __write(int fd, const void *buffer, int size)
{
  return __r_write(0, fd, buffer, size);
}
int __open(const char *path, int flags, int mode)
{
  return __r_open(0, path, flags, mode);
}
int __close(int fd)
{
  return __r_close(0, fd);
}
#endif

void exit(int status)
{
  __exit(status);
}

int strlen(char *str)
{
  int i;
  for (i = 0; str[i]; i++)
    ;
  return i;
}

int write1(int fd, unsigned char c)
{
  return __write(fd, &c, 1);
}

int putchar(int fd, int c)
{
  write1(fd, c);
  return c;
}

int nputs(int fd, char *str, int len)
{
#ifdef WRITE1
  for (; len > 0; len--)
    write1(fd, *(str++));
#else
  __write(fd, str, len);
#endif
  return 0;
}

int puts(int fd, char *str)
{
  return nputs(fd, str, strlen(str));
}

#ifdef RSHIFT
/*
 * For architecture not to be able to use shift instruction for well,
 * execute shift operation without shift instruction.
 */
unsigned long rshift1(unsigned long value)
{
  unsigned long mask, bit = 1;
  unsigned long ret = 0;
  int i;

  for (i = 0; i < sizeof(unsigned long) * 8 - 1; i++) {
    mask = bit + bit; /* Use addition for no using shift and multiplication */
    if (value & mask)
      ret |= bit;
    bit = bit + bit;
  }

  return ret;
}

unsigned long RSHIFT(unsigned long value)
{
  int num = 4;
  for (; num > 0; num--)
    value = rshift1(value);
  return value;
}
#endif

int putxval(int fd, unsigned short value, int column)
{
  char buf[17];
  char *p;

  p = buf + sizeof(buf) - 1;
  *(p--) = '\0';

  if (!value && !column)
    column++;

  while (value || column) {
    *(p--) = "0123456789abcdef"[value & 0xf];
    value >>= 4;
    if (column) column--;
  }

  puts(fd, p + 1);

  return 0;
}

int putsxval(int fd, unsigned short value)
{
  int r = putxval(fd, value, 4);
  puts(fd, "\n");
  return r;
}

#include "rnd-msp430-elf.c"
#include "rnd-mseq16.c"

int main()
{
  int fd = 1, i;
  unsigned short value;

  puts(fd, "Hello World!\n");

  puts(fd, "param set test (0000 0000 0002 0002)\n");
  value = random_param(0);   putsxval(fd, value);
  value = random_param(3);   putsxval(fd, value);
  value = random_param(0);   putsxval(fd, value);
  value = random_param(15);  putsxval(fd, value);

  puts(fd, "param unset test (000e 000e 000e)\n");
  value = random_param(0);   putsxval(fd, value);
  value = random_param(2);   putsxval(fd, value);
  value = random_param(14);  putsxval(fd, value);

  puts(fd, "seed set test (0000 0000 0001)\n");
  value = random_seed(0);     putsxval(fd, value);
  value = random_seed(1);     putsxval(fd, value);
  value = random_seed(2);     putsxval(fd, value);

  puts(fd, "value set test (0010 0008 0004)\n");
  value = random_value(8, 1 - 1); putsxval(fd, value);
  value = random_value(4, 1 - 1); putsxval(fd, value);
  value = random_value(2, 1 - 1); putsxval(fd, value);

  puts(fd, "param set (000e 0004)\n");
  value = random_param(0x55); putsxval(fd, value);
  value = random_seed(1);     putsxval(fd, value);
  puts(fd, "value get test (0002 0004 0008 0020 0200 8000 0055 00aa)\n");
  value = random_get_value(1); putsxval(fd, value);
  value = random_get_value(1); putsxval(fd, value);
  value = random_get_value(1); putsxval(fd, value);
  value = random_get_value(2); putsxval(fd, value);
  value = random_get_value(4); putsxval(fd, value);
  value = random_get_value(6); putsxval(fd, value);
  value = random_get_value(1); putsxval(fd, value);
  value = random_get_value(1); putsxval(fd, value);

  puts(fd, "value test (0054 00aa ...)\n");
  value = random_set_param_default(); putsxval(fd, value);
  value = random_set_seed_default();  putsxval(fd, value);
  for (i = 0; i < 20; i++) {
    value = random_get_value(8); putsxval(fd, value);
  }

  exit(0);
  return 0;
}
